/*
 * AdapterBase.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.client;

import ro.raducora.utils.protocol.ConnectInfo;

/**
 * @author Radu Coravu<br>
 * 
 * A base for all constructed Adapters
 */
public interface AdapterBase extends Cloneable {

  /**
   * Clone the adapter to avoid constructing it each time
   */
  public Object clone() throws CloneNotSupportedException;

  /**
   * @return </b>true if connected <b>false </b> otherwise
   */
  public boolean isConnected();

  /**
   * Connects the adapter to destination source
   * 
   * @param info
   *          connection info
   * @throws Exception
   */
  public void connect(ConnectInfo info) throws Exception;

  /**
   * Gets the connection status returned by the <code>connect</code> method
   * 
   * @return the connection status
   */
  public Object getConnectionStatus();

  /**
   * Disconnect the adapter from remote/local source
   */
  public void disconnect() throws Exception;

  /**
   * Send Object and get received response object all in one step
   */
  public Object queryObject(Object sent) throws Exception;

  /**
   * Reloads the properties from the properties sources if any if not possible
   * properties,do nothing :) called when adapter config file changes to reload
   * changes without the need of recreating the adapter
   * 
   * @throws Exception
   */
  public void reloadProperties() throws Exception;

  /**
   * Returns true if the connection is abstract(not to a real database) false
   * if the connection is made to a real database
   */
  public boolean isAbstractConnection();
}
