/*
 * AdapterManager.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.client;

import java.util.Iterator;
import java.util.Vector;

import ro.raducora.utils.io.Write;
import ro.raducora.utils.protocol.ConnectInfo;

/**
 * @author Radu Coravu<br>
 * 
 * The Manager which registers,unregisters AdapterBase implementers
 */
public class AdapterManager {

  //the list of adapters
  private Vector adapterList = new Vector(10);

  /**
   * Reload properties for all adapters. Useful when config source changes for
   * one or more adapters
   */
  public void reloadProperties() {
    Iterator adapterIter = adapterList.iterator();
    while (adapterIter.hasNext()) {
      try {
        ((AdapterBase) adapterIter.next()).reloadProperties();
      } catch (Exception exc) {
        Write.exception(exc);
      }
    }
  }

  /**
   * Registers an AdapterBase object
   * 
   * @param adapter
   *          the type of adapter to register
   * @return <b>false </b> if adapter type already registered <b>true </b>
   *         otherwise
   */
  public boolean registerAdapter(AdapterBase adapter) {
    if (adapterList.contains(adapter)) return false;
    adapterList.add(adapter);
    return true;
  }

  /**
   * De-registers an adapter
   * 
   * @param adapter
   *          the type of adapter to de-register
   * @return <b>true <b>if adapter is succesfully removed
   */
  public boolean deregisterAdapter(AdapterBase adapter) {
    if (!adapterList.remove(adapter)) return false;
    if (adapter.isConnected()) try {
      adapter.disconnect();
    } catch (Exception e) {
      Write.exception(e);
    }
    return true;
  }

  /**
   * Gets right Adapter for job. Tries all adapters until one of them does not
   * throw an Exception
   * 
   * @param info
   *          the Connection information
   * @return clone of the adapter most suited <b>null </b> if no suitable found
   */
  public synchronized AdapterBase getRightAdapter(ConnectInfo info) {
    Iterator adapterIter = adapterList.iterator();
    while (adapterIter.hasNext()) {
      try {
        AdapterBase ret = (AdapterBase) ((AdapterBase) adapterIter.next()).clone();
        ret.connect(info);
        return ret;
      } catch (Exception exception) {
        Write.exception(exception);
      }
    }
    return null;
  }

}
