/*
 * ConnectionManagerBroker.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.client.db;

import ro.raducora.utils.io.Write;

/**
 * @author Radu Coravu<br>
 * 
 * Created on 29.02.2004
 */
class ConnectionManagerBroker {

  //the interval after which to refresh old connections
  //after 4 intervals ,connections are closed to minimum
  //default set to 30 minutes
  private long cleanInterval = 1800000;

  private ConnectionPoolManager poolsManager;

  //the cleaner thread
  private Thread cleaner = null;

  /**
   * Constructor.
   * 
   * @param cleanInterval
   *          the Interval after which to clean
   */
  public ConnectionManagerBroker(int cleanInterval) {
    this();
    setCleanInterval(cleanInterval);
  }

  /**
   * Constructor. Creates the Broker
   */
  public ConnectionManagerBroker() {
    Write.Line("Connection Broker Created");
    poolsManager = new ConnectionPoolManager();
    //starts a pool cleaner thread
    cleaner = new Thread("Cleaner") {

      public void run() {
        Write.Line("Cleaner Thread Started");
        int cleanPool = 0;
        while (true) {
          try {
            try {
              //sleep 30 minutes
              sleep(cleanInterval);
            } catch (InterruptedException e) {
              //will never
              // happen
            }
            if (cleanPool == 4) {
              cleanPool = 0;
              //gets all connection managers
              //closes available connections to minimum every 120 minutes
              poolsManager.closeAvailableConnectionsToMinimumNo(cleanInterval);
            }
            cleanPool++;
            //check available connections every 30 minutes
            poolsManager.refreshConnections(cleanInterval);
          } catch (Exception e) {
            //do nothing,do not destroy thread
            Write.exception(e);
          }
        }
      }
    };
    cleaner.setDaemon(true);
    cleaner.start();
  }

  /**
   * Sets the clean interval
   * 
   * @param cleanInterval
   *          the interval after which the pool is refreshed
   */
  public void setCleanInterval(long cleanInterval) {
    this.cleanInterval = cleanInterval;
  }

  /**
   * Gets the clean interval
   */
  public long getCleanInterval() {
    return cleanInterval;
  }

  /**
   * Returns the internal Connection Pools Manager
   * 
   * @return the internal Connection Pools Manager
   */
  public ConnectionPoolManager getPoolsManager() {
    return poolsManager;
  }
}
