/*
 * ConnectionManagerTest.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.client.db;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.DriverPropertyInfo;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Savepoint;
import java.sql.Statement;
import java.util.Map;
import java.util.Properties;
import java.util.Vector;

import junit.framework.TestCase;
import ro.raducora.utils.io.Write;
import ro.raducora.utils.props.ClientDBProps;
import ro.raducora.utils.props.xml.ClientDBXMLProperties;
import ro.raducora.utils.protocol.ConnectInfo;

/**
 * @author Radu<br>
 * 
 * Created on 17.04.2004
 */
public class ConnectionManagerTest extends TestCase {

  //load driver
  //set error ,debug to true
  static {
    Write.DEBUG = true;
    Write.EXCEPTION = true;
    try {
      DriverManager.registerDriver(new Driver() {

        public Connection connect(String url, Properties info) throws SQLException {
          return new Connection() {

            public Statement createStatement() throws SQLException {
              return null;
            }

            public PreparedStatement prepareStatement(String sql) throws SQLException {
              return null;
            }

            public CallableStatement prepareCall(String sql) throws SQLException {

              return null;
            }

            public String nativeSQL(String sql) throws SQLException {

              return null;
            }

            public void setAutoCommit(boolean autoCommit) throws SQLException {

            }

            public boolean getAutoCommit() throws SQLException {

              return false;
            }

            public void commit() throws SQLException {

            }

            public void rollback() throws SQLException {

            }

            public void close() throws SQLException {

            }

            public boolean isClosed() throws SQLException {

              return false;
            }

            public DatabaseMetaData getMetaData() throws SQLException {

              return null;
            }

            public void setReadOnly(boolean readOnly) throws SQLException {

            }

            public boolean isReadOnly() throws SQLException {

              return false;
            }

            public void setCatalog(String catalog) throws SQLException {

            }

            public String getCatalog() throws SQLException {

              return null;
            }

            public void setTransactionIsolation(int level) throws SQLException {

            }

            public int getTransactionIsolation() throws SQLException {

              return 0;
            }

            public SQLWarning getWarnings() throws SQLException {

              return null;
            }

            public void clearWarnings() throws SQLException {

            }

            public Statement createStatement(int resultSetType, int resultSetConcurrency)
                throws SQLException {

              return null;
            }

            public PreparedStatement prepareStatement(String sql, int resultSetType,
                int resultSetConcurrency) throws SQLException {

              return null;
            }

            public CallableStatement prepareCall(String sql, int resultSetType,
                int resultSetConcurrency) throws SQLException {

              return null;
            }

            public Map getTypeMap() throws SQLException {

              return null;
            }

            public void setTypeMap(Map map) throws SQLException {

            }

            public void setHoldability(int holdability) throws SQLException {

            }

            public int getHoldability() throws SQLException {

              return 0;
            }

            public Savepoint setSavepoint() throws SQLException {

              return null;
            }

            public Savepoint setSavepoint(String name) throws SQLException {

              return null;
            }

            public void rollback(Savepoint savepoint) throws SQLException {

            }

            public void releaseSavepoint(Savepoint savepoint) throws SQLException {

            }

            public Statement createStatement(int resultSetType, int resultSetConcurrency,
                int resultSetHoldability) throws SQLException {

              return null;
            }

            public PreparedStatement prepareStatement(String sql, int resultSetType,
                int resultSetConcurrency, int resultSetHoldability) throws SQLException {

              return null;
            }

            public CallableStatement prepareCall(String sql, int resultSetType,
                int resultSetConcurrency, int resultSetHoldability) throws SQLException {

              return null;
            }

            public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys)
                throws SQLException {

              return null;
            }

            public PreparedStatement prepareStatement(String sql, int[] columnIndexes)
                throws SQLException {

              return null;
            }

            public PreparedStatement prepareStatement(String sql, String[] columnNames)
                throws SQLException {

              return null;
            }
          };
        }

        public boolean acceptsURL(String url) throws SQLException {
          return url.startsWith("test");
        }

        public DriverPropertyInfo[] getPropertyInfo(String url, Properties info)
            throws SQLException {
          return null;
        }

        public int getMajorVersion() {
          return 0;
        }

        public int getMinorVersion() {
          return 0;
        }

        public boolean jdbcCompliant() {
          return true;
        }
      });
    } catch (SQLException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  private String dbName = "mini";

  private String dbPass = null;

  private String dbUser = null;

  private String dbURL = "test:mini";

  /**
   * Set Up
   */
  protected void setUp() throws Exception {

  }

  /**
   * Constructor for ConnectionManagerTest.
   * 
   * @param arg0
   */
  public ConnectionManagerTest(String arg0) {
    super(arg0);
  }

  /**
   * Tests the connections cleaning.
   *  
   */
  public void testCleanConnectionPools() {
    try {
      ConnectInfo info = new ConnectInfo(dbName, dbUser, dbPass);
      //clean at 2 seconds
      ConnectionManagerBroker broker = new ConnectionManagerBroker(500);
      ClientDBProps props = new ClientDBProps() {

        public void reload() throws Exception {
        }

        public String getRealDBName(String dbname) {
          return dbname;
        }

        public String getDriverClassName(String dbname) {
          return null;
        }

        public String getDriverJarPath(String dbname) {
          return null;
        }

        public String getDriverJarFilename(String dbname) {
          return null;
        }

        public String getServerSubprotocol(String dbname) {
          return "odbc";
        }

        public String getServerHostname(String dbname) {
          return null;
        }

        public int getMinNoConn(String dbname) {
          return 2;
        }

        public int getMaxNoConn(String dbname) {
          return 30;
        }
      };
      Vector connections = new Vector();

      //make 10 connections
      for (int i = 0; i < 10; i++) {
        connections.add(broker.getPoolsManager().getConnectionPool(info,
            new ClientDBXMLProperties(), dbURL).getConnection());
      }
      //10 used connections
      assertTrue("10 used pooled connections", broker.getPoolsManager().getConnectionPool(info,
          props, dbURL).getUsedConnNumber() == 10);

      //close all 10 connections
      for (int i = 0; i < 10; i++) {
        ((Connection) connections.get(i)).close();
      }

      //10 available connections
      assertTrue("10 available pooled connections", broker.getPoolsManager().getConnectionPool(
          info, props, dbURL).getAvailConnNumber() == 10);
      Thread.sleep(4000);

      //min conn available conn after cleaning
      assertTrue(broker.getPoolsManager().getConnectionPool(info, new ClientDBXMLProperties(),
          dbURL).getAvailConnNumber()
          + " available pooled connections after cleaning",
          broker.getPoolsManager().getConnectionPool(info, new ClientDBXMLProperties(), dbURL)
              .getAvailConnNumber() == props.getMinNoConn(dbName));

      //close all connections
      broker.getPoolsManager().getConnectionPool(info, props, dbURL).closeAllConnections();

      //0 connections after cleaning
      assertTrue(
          "0 connections",
          broker.getPoolsManager().getConnectionPool(info, props, dbURL).getUsedConnNumber()
              + broker.getPoolsManager().getConnectionPool(info, props, dbURL).getAvailConnNumber() == 0);

      //clean at 2 seconds
      broker = new ConnectionManagerBroker(500);
      connections.clear();
      //make 10 connections
      for (int i = 0; i < 10; i++) {
        connections.add(broker.getPoolsManager().getConnectionPool(info, props, dbURL)
            .getConnection());
      }
      //10 used conn
      assertTrue("10 used pooled connections", broker.getPoolsManager().getConnectionPool(info,
          props, dbURL).getUsedConnNumber() == 10);

      //close 7 connections
      for (int i = 0; i < 7; i++) {
        ((Connection) connections.get(i)).close();
      }

      //7 available connections
      assertTrue("7 available pooled connections", broker.getPoolsManager().getConnectionPool(info,
          props, dbURL).getAvailConnNumber() == 7);

      //3 used connections
      assertTrue("3 used pooled connections", broker.getPoolsManager().getConnectionPool(info,
          props, dbURL).getUsedConnNumber() == 3);

      Thread.sleep(4000);

      //0 available connections
      assertTrue(broker.getPoolsManager().getConnectionPool(info, props, dbURL)
          .getAvailConnNumber()
          + " available pooled connections after cleaning", broker.getPoolsManager()
          .getConnectionPool(info, props, dbURL).getAvailConnNumber() == 0);

      //3 used connections
      assertTrue(broker.getPoolsManager().getConnectionPool(info, props, dbURL).getUsedConnNumber()
          + " used pooled connections after cleaning", broker.getPoolsManager().getConnectionPool(
          info, props, dbURL).getUsedConnNumber() == 3);

    } catch (Exception e) {
      e.printStackTrace();
      fail(e.getMessage());
    }
  }

}
