/*
 * JarDriverLoader.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.client.db;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.DriverPropertyInfo;
import java.sql.SQLException;
import java.util.Properties;

import ro.raducora.utils.io.Write;

/**
 * @author Radu Coravu <br>
 *         Loads a db driver from a jar file
 */

public class JarDriverLoader {

  //no instance allowed
  private JarDriverLoader() {
  }

  /**
   * Loads a Driver Class from a specific jar
   * 
   * @param jarPath
   *          path to jar file(not including jar file)
   * @param jarFilename
   *          (filename of jar:Eg: mysql.jar)
   * @param className
   *          (name of class to load)
   */
  public static void loadDriver(String jarPath, String jarFilename, String className)
      throws MalformedURLException, ClassNotFoundException, InstantiationException,
      IllegalAccessException, SQLException {
    String url = "";
    //if no protocol mentioned
    if (jarPath.lastIndexOf("://") == -1) {
      //gets absolute path to jar
      File jarAbsPath = new File(jarPath, jarFilename);
      url = "file:" + jarAbsPath.getAbsolutePath() + "!/";
    } else {
      //it has a protocol attached
      if (jarPath.charAt(jarPath.length()) == '\\' || jarPath.charAt(jarPath.length()) == '/') {
        if (jarFilename.charAt(jarPath.length()) == '\\'
            || jarFilename.charAt(jarPath.length()) == '/') {
          jarFilename = jarFilename.substring(0, jarFilename.length() - 1);
        }
      } else {
        if (jarFilename.charAt(jarPath.length()) != '\\'
            && jarFilename.charAt(jarPath.length()) != '/') {
          jarFilename = "/" + jarFilename;
        }
      }
      url = jarPath + jarFilename + "!/";
    }

    //make an URL out of it
    URL jarfile = new URL("jar", "", url);

    URLClassLoader cl = URLClassLoader.newInstance(new URL[] { jarfile});

    Driver d = (Driver) Class.forName(className, true, cl).newInstance();

    //register the Driver Shim
    DriverManager.registerDriver(new DriverShim(d));

    Write.Line("Load Driver from:" + jarPath + jarFilename + "//||\\\\Class Name=" + className);

  }

}

/**
 * This is a simple trick to load driver from jar dinamically using the regular
 * Class Loader
 */

class DriverShim implements Driver {

  private Driver driver;

  DriverShim(Driver d) {
    this.driver = d;
  }

  /**
   * @see java.sql.Driver#acceptsURL(java.lang.String)
   */
  public boolean acceptsURL(String u) throws SQLException {
    return this.driver.acceptsURL(u);
  }

  /**
   * @see java.sql.Driver#connect(java.lang.String, java.util.Properties)
   */
  public Connection connect(String u, Properties p) throws SQLException {
    return this.driver.connect(u, p);
  }

  /**
   * @see java.sql.Driver#getMajorVersion()
   */
  public int getMajorVersion() {
    return this.driver.getMajorVersion();
  }

  /**
   * @see java.sql.Driver#getMinorVersion()
   */
  public int getMinorVersion() {
    return this.driver.getMinorVersion();
  }

  /**
   * @see java.sql.Driver#getPropertyInfo(java.lang.String,
   *      java.util.Properties)
   */
  public DriverPropertyInfo[] getPropertyInfo(String u, Properties p) throws SQLException {
    return this.driver.getPropertyInfo(u, p);
  }

  /**
   * @see java.sql.Driver#jdbcCompliant()
   */
  public boolean jdbcCompliant() {
    return this.driver.jdbcCompliant();
  }
}
