/*
 * ClientTCP.java
 *
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.

 * The DBAbstract Project is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with the DBAbstract Project; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */package ro.raducora.connection.client.tcp;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.zip.GZIPInputStream;

import ro.raducora.utils.protocol.RemoteException;

/**
 * 
 * @author Radu Coravu
 *
 * <br>Created on 06.06.2004
 * <br>This is a ClientTCP abstraction which communicates by streams. 
 */
public class ClientTCP {

    private ObjectInputStream input = null;

    private ObjectOutputStream output = null;

    //gunzip input
    private boolean gunzipInput = false;

    /**
     * Set ObjectInputStream and ObjectOutputStream
     * 
     * @param input
     *          the Object Input Stream
     * @param output
     *          the Object Output Stream
     */
    public ClientTCP(ObjectInputStream input, ObjectOutputStream output,
            boolean gunzipInput) {
        this.input = input;
        this.output = output;
        this.gunzipInput = gunzipInput;
    }

    /**
     * Get an object from ObjectInputStream
     * 
     * @return the read object
     * @throws IOException
     * @throws ClassNotFoundException
     * @throws RemoteException
     */
    public synchronized Object getObject() throws IOException,
            ClassNotFoundException, RemoteException {
        Object object = readObject();
        if (object instanceof RemoteException)
                throw ((RemoteException) object);
        return object;
    }

    /**
     * Send an object to server
     * 
     * @param sent
     *          the object to send
     * @throws IOException
     */
    public void sendObject(Object sent) throws IOException {
        output.writeObject(sent);
        output.flush();
    }

    /**
     * Close object streams
     *  
     */
    public void closeConnection() {
        try {
            output.close();
        } catch (IOException e1) {
        }
        try {
            input.close();
        } catch (IOException e) {
        }
    }

    private Object readObject() throws IOException, ClassNotFoundException {
        if (gunzipInput) {
            GZIPInputStream gunzipInputStr = new GZIPInputStream(input);
            ObjectInputStream gunzipStream = new ObjectInputStream(
                    gunzipInputStr);
            Object obj = gunzipStream.readObject();
            return obj;
        } else {
            return input.readObject();
        }
    }
}
