/*
 * ServerLauncher.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.server;

import java.util.Enumeration;
import java.util.Vector;

import ro.raducora.utils.io.Write;

/**
 * @author Radu Coravu
 * 
 * <br>
 * A Server Launcher which registers ,unregisters Server Implementators
 */
public class ServerLauncher {

  private static Vector servers = new Vector(10);

  private ServerLauncher() {
  }

  /**
   * Tries to reload properties for all servers.
   *  
   */
  public static boolean reloadAll() {
    boolean allReloaded = true;
    for (int i = 0; i < servers.size(); i++) {
      try {
        Server curr = (Server) servers.elementAt(i);
        curr.reload();
      } catch (Exception e) {
        Write.exception(e);
        allReloaded = false;
      }
    }
    return allReloaded;
  }

  /**
   * Registers a Server
   * 
   * @param server
   *          The server to register
   * @return true If the server was registered false otherwise
   */
  public static boolean registerServer(Server server) {
    if (servers.contains(server)) { return false; }
    servers.add(server);
    return true;
  }

  /**
   * Registers an array of servers
   * 
   * @param serverArray
   *          the server array to register
   * @return true if the servers were registered
   */
  public static boolean registerServers(Server[] serverArray) {
    for (int i = 0; i < serverArray.length; i++) {
      registerServer(serverArray[i]);
    }
    return true;
  }

  /**
   * Deregisters a Server
   * 
   * @param server
   *          the server to de-register
   * @return true if the server was de-registered
   */
  public static boolean deRegisterServer(Server server) {
    if (servers.contains(server)) {
      servers.remove(server);
      return true;
    }
    return false;
  }

  /**
   * De-register all Servers
   * 
   * @return true
   */
  public static boolean deRegisterAll() {
    servers.removeAllElements();
    return true;
  }

  /**
   * Starts all registered Servers
   */
  public static void startAll() {
    Enumeration enum = servers.elements();
    while (enum.hasMoreElements()) {
      ((Server) enum.nextElement()).startServer();
    }
  }

  /**
   * Stops all registered Servers
   */
  public static void stopAll() {
    Enumeration enum = servers.elements();
    while (enum.hasMoreElements()) {
      ((Server) enum.nextElement()).stopServer();
    }
  }

  /**
   * Starts a certain registered Server
   */
  public static void startServer(Server server) {
    Enumeration enum = servers.elements();
    while (enum.hasMoreElements()) {
      Server curr = ((Server) enum.nextElement());
      if (curr.equals(server)) {
        curr.startServer();
        return;
      }
    }
  }

  /**
   * Stops a certain registered Server
   */
  public static void stopServer(Server server) {
    Enumeration enum = servers.elements();
    while (enum.hasMoreElements()) {
      Server curr = ((Server) enum.nextElement());
      if (curr.equals(server)) {
        curr.stopServer();
        return;
      }
    }
  }

  /**
   * Starts all registered Servers except the one sent as parameter
   * 
   * @param server
   *          the exception
   */
  public static void startServersExcept(Server server) {
    Enumeration enum = servers.elements();
    while (enum.hasMoreElements()) {
      Server curr = ((Server) enum.nextElement());
      if (!curr.equals(server)) {
        curr.startServer();
      }
    }
  }

  /**
   * Stops registered Servers except the one sent as parameter
   * 
   * @param server
   *          the exception
   */
  public static void stopServersExcept(Server server) {
    Enumeration enum = servers.elements();
    while (enum.hasMoreElements()) {
      Server curr = ((Server) enum.nextElement());
      if (!curr.equals(server)) {
        curr.stopServer();
      }
    }
  }

}
