/*
 * TelnetServer.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.server.telnet;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import ro.raducora.connection.server.Server;
import ro.raducora.utils.io.Write;

/**
 * @author Radu Coravu
 * 
 * <br>
 * A Telnet Server used to reload porperties files without stopping the
 * DBAbstract Server
 */
public class TelnetServer extends Thread implements Server {

  private ServerSocket server = null;

  //port the telnet server runs on
  private int PORT = 16666;

  private boolean stop = false;

  /**
   * Constructor.
   * 
   * @param PORT
   *          the telnet server PORT
   */
  public TelnetServer(int PORT) {
    this.PORT = PORT;
    this.setDaemon(false);
    this.setName("Telnet Server");
  }

  /**
   * Runs the Server.
   */
  public void run() {
    try {
      server = new ServerSocket(PORT);
      server.setSoTimeout(5000);
    } catch (IOException e) {
      Write.exception(e);
      return;
    }

    while (!stop) {
      try {
        Socket sock = server.accept();
        sock.setKeepAlive(true);
        Thread thread = new TelnetThread(sock.getInputStream(), sock.getOutputStream());
        thread.setName("Telnet Thread");
        thread.run();
      } catch (IOException e1) {
      }
    }
  }

  /**
   * Start the Server
   */
  public synchronized void startServer() {
    stop = false;
    super.start();
  }

  /**
   * Stop the Server
   */
  public void stopServer() {
    stop = true;
  }

  /**
   * Do nothing
   */
  public void reload() throws Exception {
  }

}
