/*
 * TelnetThread.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.server.telnet;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import ro.raducora.connection.server.ServerLauncher;
import ro.raducora.utils.io.Write;

/**
 * @author Radu Coravu
 * 
 * <br>
 * A Telnet Thread
 *  
 */
public class TelnetThread extends Thread {

  private BufferedOutputStream output = null;

  private BufferedInputStream input = null;

  /**
   * Constructor.
   * 
   * @param input
   *          the input stream
   * @param output
   *          the output stream
   */
  public TelnetThread(InputStream input, OutputStream output) {
    this.output = new BufferedOutputStream(output);
    this.input = new BufferedInputStream(input);
    this.setDaemon(true);
  }

  /**
   * Runs the Server Thread.
   */
  public void run() {
    try {
      Write.Line("Telnet Thread Started");
      byte[] buf = new byte[1];
      String command = "";
      writeString("Welcome to the Telnet Connector\r\n");
      writeString("Command:");
      while (true) {
        input.read(buf, 0, buf.length);
        command += new String(buf);

        //backspace
        if (buf[0] == 8) {
          if (command.length() > 2) {
            command = command.substring(0, command.length() - 2);
          }
        }
        if (buf[0] == 10 || buf[0] == 63) {
          try {
            decideOn(command);
          } catch (Exception e1) {
            Write.exception(e1);
            writeString(e1.getMessage());
          }
          command = new String();
          writeString("Command:");
        }

      }
    } catch (IOException e) {
      closeConnection();
      return;
    }
  }

  private void closeConnection() {
    try {
      output.close();
    } catch (IOException e1) {
    }
    try {
      input.close();
    } catch (IOException e) {
    }
  }

  private void decideOn(String command) throws Exception {
    String cmd = command.trim();
    if (cmd.equalsIgnoreCase("?") || cmd.equalsIgnoreCase("help")) {
      displayHelp();
      return;
    }

    if (cmd.toUpperCase().startsWith("RELOAD")) {
      ServerLauncher.reloadAll();
      writeString("Properties reloaded\r\n");
    } else if (cmd.toUpperCase().startsWith("DEBUG")) {
      Write.DEBUG = true;
      writeString("Debug Mode Started\r\n");
    } else if (cmd.toUpperCase().startsWith("UNDEBUG")) {
      Write.DEBUG = false;
      writeString("Debug Mode Stopped\r\n");
    } else if (cmd.toUpperCase().startsWith("EXCEPTIONS")) {
      Write.EXCEPTION = true;
      writeString("Exceptions output started\r\n");
    } else if (cmd.toUpperCase().startsWith("NOEXCEPTIONS")) {
      Write.EXCEPTION = false;
      writeString("Exceptions output stopped\r\n");
    } else {
      writeString("Enter ? for help\r\n");
    }

  }

  private void displayHelp() throws IOException {
    writeString("\r\nCommand Usage:\r\n");
    writeString("? --> displays this menu\r\n");
    writeString("help --> displays this menu\r\n");
    writeString("reload -->reloads modified config files without stopping the server\r\n");
    writeString("debug -->start debug mode for server\r\n");
    writeString("undebug -->stop debug mode for server\r\n");
    writeString("exceptions -->show exceptions for server\r\n");
    writeString("noexceptions -->hide exceptions for server\r\n");
  }

  private void writeString(String str) throws IOException {
    output.write(str.getBytes());
    output.flush();
  }

}
