/**
 * Created on 06.12.2003 by Radu Coravu
 */
package ro.raducora.samples;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import ro.raducora.connection.client.AdapterManager;
import ro.raducora.connection.client.db.ClientDBAdapter;
import ro.raducora.connection.client.tcp.ClientTCPAdapter;
import ro.raducora.connection.server.Server;
import ro.raducora.connection.server.ServerLauncher;
import ro.raducora.connection.server.tcp.ServerTCPAdapter;
import ro.raducora.connection.server.telnet.TelnetServer;
import ro.raducora.utils.encryption.key.RC4EncryptionKey;
import ro.raducora.utils.io.Read;
import ro.raducora.utils.io.Write;
import ro.raducora.utils.props.ServerProps;
import ro.raducora.utils.props.xml.ClientDBXMLProperties;
import ro.raducora.utils.props.xml.ClientTCPXMLProperties;
import ro.raducora.utils.props.xml.ServerXMLProperties;

/**
 * @author Radu Coravu
 * 
 * <br>
 * A Sample Server with basic configuration, uses 2 client adapters(DB and TCP), 
 * has a TELNET Server, uses RC4 encryption.
 */
public class SampleServer {

  /**
   * @param args
   *          (port,max_no_clients[,tcp_props_file,db_props_file])
   * @throws ParserConfigurationException
   * @throws SAXException
   */
  public static void main(String[] args) throws ParserConfigurationException, SAXException {
    try {
      int port = 11111;
      String conftcp = "./conf/ClientTCP.xml";
      String confdb = "./conf/ClientDB.xml";
      String confserv = "./conf/Server.xml";
      if (args.length == 1) {
        port = Integer.parseInt(args[0]);
      } else {
        if (args.length == 4) {
          port = Integer.parseInt(args[0]);
          conftcp = args[1];
          confdb = args[2];
          confserv = args[3];
        } else {
          System.out
              .println("Command Usage: java ro.raducora.samples.SampleServer port \n"
                  + "Command Usage: java ro.raducora.samples.SampleServer port configtcp.xml configdb.xml configserv.xml");
          port = Integer.parseInt(Read.Line("Enter server port:"));
        }
      }
      Write.DEBUG = false;
      Write.EXCEPTION = false;
      AdapterManager adapterManager = new AdapterManager();
      adapterManager.registerAdapter(new ClientDBAdapter(new ClientDBXMLProperties(confdb)));
      adapterManager.registerAdapter(new ClientTCPAdapter(new ClientTCPXMLProperties(conftcp),
          new RC4EncryptionKey(), new RC4EncryptionKey()));

      //            AdapterManager.registerAdapter(
      //                new ClientTCPAdapter(
      //                    new ClientTCPXMLProperties(conftcp),
      //                    new DefaultEncryptionKey(),
      //                    new DefaultEncryptionKey()));
      ServerProps serverProps = new ServerXMLProperties(confserv);
      Server rc4server = new ServerTCPAdapter(serverProps,adapterManager, port, new RC4EncryptionKey(),
          new RC4EncryptionKey());
      //            Server defaultserver =
      //                new ServerTCPAdapter(
      //                    port,
      //                    10000,
      //                    new DefaultEncryptionKey(),
      //                    new DefaultEncryptionKey());

      System.out.println("Server started on port " + port);
      Server telnet = new TelnetServer(port + 1);

      ServerLauncher.registerServer(telnet);
      ServerLauncher.registerServer(rc4server);
      //            ServerLauncher.registerServer(defaultserver);
      ServerLauncher.startAll();

      while (true) {//!Read.Line("Stop?").equalsIgnoreCase("Y")) {
        Thread.sleep(10000);
      }
      //  ServerLauncher.stopAll();
    } catch (IOException e) {
      e.printStackTrace();
    } catch (InterruptedException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }
}
