/*
 * DBConnectTab.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.samples.swing;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

/**
 * @author Radu Coravu<br>
 * 
 * Created on 04.04.2004
 */
public class DBConnectTab extends JPanel {

  private Connection connection;

  private JTabbedPane parent;

  private JLabel userLabel;

  private JLabel passLabel;

  private JLabel dbLabel;

  private JLabel hostLabel;

  private JTextField userTextField;

  private JPasswordField passField;

  private JTextField dbTextField;

  private JTextField hostTextField;

  private JTextArea statusArea;

  private JScrollPane statusPane;

  private JButton connectButton;

  private DBTableTab tab;

  /**
   * Constructor
   */
  public DBConnectTab() {
    setLayout(new BorderLayout());
    JPanel inner = new JPanel(new GridBagLayout());

    GridBagConstraints constr = new GridBagConstraints();

    constr.gridx = 0;
    constr.gridy = 0;
    constr.gridwidth = 1;
    constr.insets = new Insets(5, 5, 5, 5);
    userLabel = new JLabel("Username:");
    inner.add(userLabel, constr);

    constr.gridx = 1;
    constr.gridy = 0;
    constr.gridwidth = 1;
    constr.fill = GridBagConstraints.HORIZONTAL;
    userTextField = new JTextField();
    userTextField.setPreferredSize(new Dimension(100, 21));
    inner.add(userTextField, constr);

    constr.gridx = 0;
    constr.gridy = 1;
    constr.gridwidth = 1;
    passLabel = new JLabel("Password:");
    inner.add(passLabel, constr);

    constr.gridx = 1;
    constr.gridy = 1;
    constr.gridwidth = 1;
    constr.fill = GridBagConstraints.HORIZONTAL;
    passField = new JPasswordField();
    passField.setPreferredSize(new Dimension(100, 21));
    inner.add(passField, constr);

    constr.gridx = 0;
    constr.gridy = 2;
    constr.gridwidth = 1;
    hostLabel = new JLabel("Choose Host:");
    inner.add(hostLabel, constr);

    constr.gridx = 1;
    constr.gridy = 2;
    constr.gridwidth = 1;
    constr.fill = GridBagConstraints.HORIZONTAL;
    hostTextField = new JTextField("127.0.0.1");
    hostTextField.setPreferredSize(new Dimension(100, 21));
    inner.add(hostTextField, constr);

    constr.gridx = 0;
    constr.gridy = 3;
    constr.gridwidth = 1;
    dbLabel = new JLabel("Database Name:");
    inner.add(dbLabel, constr);

    constr.gridx = 1;
    constr.gridy = 3;
    constr.gridwidth = 1;
    constr.fill = GridBagConstraints.HORIZONTAL;
    dbTextField = new JTextField();
    dbTextField.setPreferredSize(new Dimension(100, 21));
    inner.add(dbTextField, constr);

    constr.gridx = 1;
    constr.gridy = 4;
    constr.gridwidth = 1;
    constr.fill = GridBagConstraints.HORIZONTAL;
    connectButton = new JButton("Connect");
    inner.add(connectButton, constr);

    constr.gridx = 1;
    constr.gridy = 5;
    constr.gridwidth = 3;
    constr.gridheight = 3;
    constr.fill = GridBagConstraints.BOTH;
    statusArea = new JTextArea("Status:");
    statusPane = new JScrollPane(statusArea);

    JPanel innerBorder=new JPanel(new BorderLayout());
    innerBorder.add(inner,BorderLayout.NORTH);
    add(innerBorder, BorderLayout.WEST);
    add(statusPane, BorderLayout.CENTER);
    initActions();
  }

  /**
   * Init Actions
   */
  private void initActions() {
    try {
      Class.forName("ro.raducora.utils.jdbc.Driver");
    } catch (ClassNotFoundException e5) {
      e5.printStackTrace();
      statusArea.append("\nCould not load driver!");
    }

    dbTextField.addKeyListener(new KeyAdapter() {

      public void keyTyped(KeyEvent e) {
        if (e.getKeyChar() == KeyEvent.VK_ENTER) {
          connectAction();
        }
      }
    });

    connectButton.addActionListener(new ActionListener() {

      public void actionPerformed(ActionEvent e) {
        connectAction();
      }
    });

  }

  /**
   * Connect or Disconnect
   */
  public void connectAction() {
    if (connectButton.getText().equalsIgnoreCase("Connect")) {
      try {
        connection = DriverManager.getConnection("jdbc:jabs://" + hostTextField.getText() + "/"
            + dbTextField.getText(), userTextField.getText(), new String(passField.getPassword()));
        parent.setSelectedIndex(1);
        tab.setConnection(connection);
        connectButton.setText("Disconnect");
      } catch (SQLException e1) {
        statusArea.append("\nConnection Failed:" + e1.getMessage());
      }

    } else {
      try {
        connectButton.setText("Connect");
        connection.close();
        statusArea.append("\n" + "Disconnected");
      } catch (Exception e1) {
        e1.printStackTrace();
        statusArea.append("\n" + "Disconnect Failed:" + e1.getMessage());
      }

    }
  }

  /**
   * Sets the parent JTabbedPane
   */
  public void setParentAndChild(JTabbedPane parent, DBTableTab tab) {
    this.parent = parent;
    this.tab = tab;
  }

  /**
   * Get the status area
   * 
   * @return the Status Area
   */
  public JTextArea getStatus() {
    return statusArea;
  }

  /**
   * Test
   */
  public static void main(String[] args) {
    JFrame frame = new JFrame("Test");
    frame.getContentPane().add(new DBConnectTab());
    frame.setSize(new Dimension(200, 300));
    frame.pack();
    frame.show();
  }
}
