/*
 * DBTableTab.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.samples.swing;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;

/**
 * @author Radu Coravu<br>
 * 
 * Created on 04.04.2004
 */
public class DBTableTab extends JPanel {

  private JTabbedPane parent;

  private JTextArea status;

  private JTable dbTable;

  private JScrollPane dbScrollPane;

  private JLabel queryLabel;

  private JComboBox queryCombo;

  private Connection connection;

  /**
   * Constructor
   */
  public DBTableTab() {
    setLayout(new BorderLayout());

    JPanel inner = new JPanel(new GridBagLayout());

    GridBagConstraints constr = new GridBagConstraints();

    constr.gridx = 0;
    constr.gridy = 0;
    constr.gridwidth = 1;
    constr.insets = new Insets(5, 5, 5, 5);
    queryLabel = new JLabel("Enter Query:");
    inner.add(queryLabel, constr);

    constr.gridx = 1;
    constr.gridy = 0;
    constr.gridwidth = 2;
    constr.fill = GridBagConstraints.HORIZONTAL;
    queryCombo = new JComboBox();
    queryCombo.setPreferredSize(new Dimension(300, 21));
    queryCombo.setEditable(true);
    inner.add(queryCombo, constr);

    constr.gridx = 0;
    constr.gridy = 1;
    constr.gridwidth = 3;
    constr.gridheight = 3;
    constr.fill = GridBagConstraints.BOTH;
    dbTable = new JTable();
    dbTable.setModel(new RSTableModel());
    dbTable.setEnabled(false);
    dbScrollPane = new JScrollPane(dbTable);

    add(inner, BorderLayout.NORTH);
    add(dbScrollPane, BorderLayout.CENTER);
    initActions();
  }

  /**
   * Sets the JPane's parent
   */
  public void setParentAndStatus(JTabbedPane parent, JTextArea status) {
    this.parent = parent;
    this.status = status;
  }

  /**
   * Sets a new connection
   */
  public void setConnection(Connection connection) {
    this.connection = connection;
  }

  /**
   *  
   */
  public void initActions() {
    queryCombo.getEditor().getEditorComponent().addKeyListener(new KeyAdapter() {

      public void keyTyped(KeyEvent e) {
        if (e.getKeyChar() == KeyEvent.VK_ENTER) {
          if (connection != null) {
            String query = (String) queryCombo.getSelectedItem();
            if (query.toUpperCase().startsWith("SELECT") || query.toUpperCase().startsWith("SHOW")) {
              ResultSet resultset = null;
              try {
                resultset = connection.createStatement().executeQuery(query);
                RSTableModel model = (RSTableModel) dbTable.getModel();
                model.addResultSet(resultset);
              } catch (SQLException e1) {
                status.append("\nDisconnected:\n" + e1.getMessage());
                parent.setSelectedIndex(0);
                ((DBConnectTab) parent.getSelectedComponent()).connectAction();
              }
            } else {
              try {
                //for insert update....
                int modified = connection.createStatement().executeUpdate(query);
                status.append("\nNumber of modified rows:" + modified);
                parent.setSelectedIndex(0);
              } catch (SQLException e1) {
                status.append("\nDisconnected:\n" + e1.getMessage());
                parent.setSelectedIndex(0);
                ((DBConnectTab) parent.getSelectedComponent()).connectAction();
              }

            }
            boolean addItem = true;
            for (int i = 0; i < queryCombo.getItemCount(); i++) {
              if (queryCombo.getItemAt(i).equals(query)) {
                addItem = false;
              }
            }
            if (addItem) {
              queryCombo.addItem(query);
            }

          }

        }
      }

    });
  }

  /**
   * Test
   */
  public static void main(String[] args) {
    JFrame frame = new JFrame("Test");
    frame.getContentPane().add(new DBTableTab());
    frame.setSize(new Dimension(200, 300));
    frame.pack();
    frame.show();
  }
}
