/*
 * DefaultEncryptionKey.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.encryption.key;

/**
 * @author Radu Coravu
 *  
 */

public class DefaultEncryptionKey extends StreamEncryptionKey {

  //the real key is a byte array
  private byte[] key;

  private boolean explicitlySet = false;

  private int ptrDecr = 0;

  /**
   * Sets the internal key with help from current system time
   */
  public DefaultEncryptionKey() {
    this(System.currentTimeMillis());
    reset();
  }

  /**
   * Constructor that takes a "long" as argument
   */
  public DefaultEncryptionKey(long longkey) {
    key = new byte[8];
    key[0] = (byte) (longkey);
    key[1] = (byte) (longkey >>> 8);
    key[2] = (byte) (longkey >>> 16);
    key[3] = (byte) (longkey >>> 24);
    key[4] = (byte) (longkey >>> 32);
    key[5] = (byte) (longkey >>> 40);
    key[6] = (byte) (longkey >>> 48);
    key[7] = (byte) (longkey >>> 54);
    explicitlySet = true;
    reset();
  }

  /**
   * Constructor that takes a String as argument
   */
  public DefaultEncryptionKey(String strkey) {
    key = strkey.getBytes();
    reset();
    explicitlySet = true;
  }

  /**
   * @return true if key is explicitly set
   */
  public boolean isExplicitlySet() {
    return explicitlySet;
  }

  /**
   * Set Key to initial state.
   */
  public void reset() {
    ptrDecr = 0;
    explicitlySet = false;
  }

  /**
   * Set Key
   * 
   * @param key
   *          the key to set.
   */
  public void setKey(String key) {
    reset();
    this.key = key.getBytes();
    explicitlySet = true;
  }

  /**
   * Takes an array of bytes and encodes it with key
   * 
   * @param toEncrypt
   *          byte array to encrypt
   * @param offset
   *          where to begin
   * @param length
   *          where to end
   * 
   * @return the encrypted byte[]
   */
  public synchronized byte[] encrypt(byte[] toEncrypt, int offset, int length) {
    return crypt(toEncrypt, offset, length);
  }

  /**
   * 
   * Takes an array of bytes and decrypts it
   * 
   * @param toDecrypt
   *          byte array to decrypt
   * @param offset
   *          where to begin
   * @param length
   *          where to end
   * 
   * @return the decrypted byte[]
   */
  public synchronized byte[] decrypt(byte[] toDecrypt, int offset, int length) {
    return crypt(toDecrypt, offset, length);
  }

  private byte[] crypt(byte[] toCrypt, int offset, int length) {
    byte[] ret = new byte[length + offset];
    for (int i = offset; i < length + offset; i++) {
      ret[i] = (byte) (toCrypt[i] ^ key[ptrDecr]);
      ptrDecr = (++ptrDecr) % key.length;
    }
    return ret;
  }
}
