/*
 * RC4EncryptionKey.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.encryption.key;

import java.security.InvalidKeyException;

import ro.raducora.utils.encryption.RC4;

/**
 * @author Radu Coravu
 *  
 */
public class RC4EncryptionKey extends StreamEncryptionKey {

  private byte[] key = null;

  private boolean explicitlySet = false;

  private RC4 encryptor = null;

  /**
   * Init Ciphers
   */
  public void reset() {
    explicitlySet = false;
    encryptor = new RC4();
  }

  /**
   * Set the key.
   */
  public void setKey(String key) throws InvalidKeyException {
    reset();
    this.key = key.getBytes();
    encryptor.engineInitEncrypt(this.key);
    explicitlySet = true;
  }

  /**
   * @return true if key explicitly set.
   */
  public boolean isExplicitlySet() {
    return explicitlySet;
  }

  /**
   * Encrypt.
   */
  public synchronized byte[] encrypt(byte[] toEncrypt, int offset, int length) {
    byte[] ret = new byte[length + offset];
    encryptor.rc4(toEncrypt, offset, length - offset, ret, offset);
    return ret;
  }

  /**
   * Decrypt.
   */
  public synchronized byte[] decrypt(byte[] toDecrypt, int offset, int length) {
    byte[] ret = new byte[length + offset];
    encryptor.rc4(toDecrypt, offset, length - offset, ret, offset);
    return ret;
  }
}
