/*
 * StreamEncryptionKey.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.encryption.key;

/**
 * @author Radu Coravu
 *  
 */

public abstract class StreamEncryptionKey implements Cloneable {

  /**
   * Clone Supported.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone();
  }

  /**
   * Takes a string and makes a key
   * 
   * @param key
   *          the String
   */
  public abstract void setKey(String key) throws Exception;

  /**
   * @return True if the key was initialized with a non-default constructor or
   *         with the setKey method and false if the key was initialised with
   *         the default constructor
   */
  public abstract boolean isExplicitlySet();

  /**
   * Resets the encryption/decryption process,initializez variables,etc so that
   * the key can be used as a brand new one
   */
  public abstract void reset() throws Exception;

  /**
   * Takes an array of bytes and encodes it with key
   * 
   * @param toEncrypt
   *          bytes to encrypt
   * @param offset
   *          where to begin
   * @param length
   *          how many bytes to process
   * @return the encrypted array(could be different size than input because of
   *         padding)
   */
  public abstract byte[] encrypt(byte[] toEncrypt, int offset, int length);

  /**
   * 
   * Takes an array of bytes and decrypts it
   * 
   * @param toDecrypt
   *          bytes to decrypt
   * @param offset
   *          where to begin
   * @param length
   *          how many bytes to process
   * @return the decrypted array(could be different size than input because of
   *         padding removal)
   */
  public abstract byte[] decrypt(byte[] toDecrypt, int offset, int length);

}
