/*
 * DecryptedInputStream.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.encryption.stream;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;

import ro.raducora.utils.encryption.key.StreamEncryptionKey;

/**
 * @author Radu Coravu
 *  
 */

public class DecryptedInputStream extends FilterInputStream {

  //the internal EncryptionKey
  private StreamEncryptionKey key = null;

  private DecryptedInputStream(InputStream arg0) {
    super(arg0);
  }

  /**
   * Constructor.
   * 
   * @param input
   *          the InputStream to decrypt
   * @param key
   *          the key to use for decryption
   */
  public DecryptedInputStream(InputStream input, StreamEncryptionKey key) {
    this(input);
    this.key = key;
  }

  /**
   * Read the byte[].
   */
  public int read(byte[] b, int off, int len) throws IOException {
    int ret = super.read(b, off, len);
    byte[] ptr = key.decrypt(b, off, ret);
    System.arraycopy(ptr, off, b, off, ptr.length + off);
    return ret;
  }

  /**
   * Read.
   */
  public int read(byte[] b) throws IOException {
    return this.read(b, 0, b.length);
  }

  /**
   * Read one byte.
   */
  public int read() throws IOException {
    byte[] ret = key.decrypt(new byte[] { (byte) super.read()}, 0, 1);
    return (int) ret[0];
  }

}
