/*
 * EncryptedOutputStream.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.encryption.stream;

import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import ro.raducora.utils.encryption.key.StreamEncryptionKey;

/**
 * @author Radu Coravu
 *  
 */

public class EncryptedOutputStream extends FilterOutputStream {

  //the key to use for encription
  private StreamEncryptionKey key = null;

  private EncryptedOutputStream(OutputStream arg0) {
    super(arg0);
  }

  /**
   * Constructor.
   * 
   * @param output
   *          the OutputStream to listen to
   * @param key
   *          the key to use in encripting the output
   */
  public EncryptedOutputStream(OutputStream output, StreamEncryptionKey key) {
    this(output);
    this.key = key;
  }

  /**
   * Write byte[].
   */
  public void write(byte[] b) throws IOException {
    this.write(b, 0, b.length);
  }

  /**
   * Write one byte.
   */
  public void write(int b) throws IOException {
    this.write(new byte[] { (byte) b}, 0, 1);
  }

  /**
   * Write byte[].
   */
  public void write(byte[] b, int off, int len) throws IOException {
    byte[] toWrite = key.encrypt(b, off, len);
    super.out.write(toWrite, off, toWrite.length);
  }
}
