/*
 * Write.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.io;

import java.io.PrintStream;

/**
 * @author Radu Coravu
 *  
 */
public class Write {

  /**
   * <code>DEBUG</code> Flag
   */
  public static boolean DEBUG = false;

  /**
   * <code>EXCEPTION</code> Flag
   */
  public static boolean EXCEPTION = false;

  private static PrintStream out = System.out;

  private static PrintStream err = System.err;

  /**
   * Set the error stream
   * 
   * @param err
   *          The Error Stream to write exceptions to
   */
  public static void setErrorStream(PrintStream err) {
    Write.err = err;
  }

  /**
   * Set the debug stream
   * 
   * @param out
   *          The Debug Stream to write exceptions to
   */
  public static void setDebugStream(PrintStream out) {
    Write.out = out;
  }

  /**
   * Shortcut to write to console with new line
   * 
   * @param line
   *          string to be written to stdout
   */
  public static void Line(String line) {
    if (DEBUG) {
      out.println("[" + Thread.currentThread().getName() + "]:: " + line);
    }
  }

  /**
   * Shortcut to write to console without new line
   * 
   * @param it
   *          string to be written to stdout
   */
  public static void It(String it) {
    if (DEBUG) {
      out.print("[" + Thread.currentThread().getName() + "]:: " + it);
    }
  }

  /**
   * Shortcut to write exception to error stream
   * 
   * @param exception
   *          exception to be written to stderr
   */
  public static void exception(Exception exception) {
    if (EXCEPTION) {
      out.print("[" + Thread.currentThread().getName() + "]:: ");
      exception.printStackTrace(err);
    }
  }

}