/*
 * WrapperConnection.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.jdbc;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Savepoint;
import java.sql.Statement;
import java.util.Map;

import ro.raducora.connection.client.AdapterBase;
import ro.raducora.utils.protocol.ConnectInfo;

/**
 * @author Radu<br>
 * 
 * Created on 06.03.2004
 */
public class WrapperConnection implements Connection {

  private AdapterBase adapter = null;

  private ConnectInfo info = null;

  /**
   * @param adapter
   *          The Adapter to use for connection
   * @param info
   *          Connection Info
   */
  public WrapperConnection(AdapterBase adapter, ConnectInfo info) throws SQLException {
    this.adapter = adapter;
    this.info = info;
    if (!adapter.isConnected()) {
      try {
        adapter.connect(info);
      } catch (Exception e) {
        throw new SQLException(e.getMessage());
      }
    }

  }

  /**
   * @see java.sql.Connection#createStatement()
   */
  public Statement createStatement() throws SQLException {
    if (adapter.isConnected()) {
      return new WrapperStatement(adapter, info);
    } else {
      throw new SQLException("Cannot reopen connection");
    }
  }

  /**
   * @see java.sql.Connection#prepareStatement(java.lang.String)
   */
  public PreparedStatement prepareStatement(String sql) throws SQLException {
    if (adapter.isConnected()) {
      return new WrapperPreparedStatement(createStatement(), sql);
    } else {
      throw new SQLException("Cannot reopen connection");
    }
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#prepareCall(java.lang.String)
   */
  public CallableStatement prepareCall(String sql) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#nativeSQL(java.lang.String)
   */
  public String nativeSQL(String sql) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#setAutoCommit(boolean)
   */
  public void setAutoCommit(boolean autoCommit) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#getAutoCommit()
   */
  public boolean getAutoCommit() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#commit()
   */
  public void commit() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#rollback()
   */
  public void rollback() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * @see java.sql.Connection#close()
   */
  public void close() throws SQLException {
    if (adapter.isConnected()) {
      try {
        adapter.disconnect();
      } catch (Exception e) {
        throw new SQLException(e.getMessage());
      }
    }
  }

  /**
   * @see java.sql.Connection#isClosed()
   */
  public boolean isClosed() throws SQLException {
    return !adapter.isConnected();
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#getMetaData()
   */
  public DatabaseMetaData getMetaData() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#setReadOnly(boolean)
   */
  public void setReadOnly(boolean readOnly) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * @see java.sql.Connection#isReadOnly()
   */
  public boolean isReadOnly() throws SQLException {
    return true;
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#setCatalog(java.lang.String)
   */
  public void setCatalog(String catalog) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#getCatalog()
   */
  public String getCatalog() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#setTransactionIsolation(int)
   */
  public void setTransactionIsolation(int level) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#getTransactionIsolation()
   */
  public int getTransactionIsolation() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#getWarnings()
   */
  public SQLWarning getWarnings() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#clearWarnings()
   */
  public void clearWarnings() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#createStatement(int, int)
   */
  public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#prepareStatement(java.lang.String, int, int)
   */
  public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency)
      throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#prepareCall(java.lang.String, int, int)
   */
  public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency)
      throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#getTypeMap()
   */
  public Map getTypeMap() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#setTypeMap(java.util.Map)
   */
  public void setTypeMap(Map map) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#setHoldability(int)
   */
  public void setHoldability(int holdability) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#getHoldability()
   */
  public int getHoldability() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#setSavepoint()
   */
  public Savepoint setSavepoint() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#setSavepoint(java.lang.String)
   */
  public Savepoint setSavepoint(String name) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#rollback(java.sql.Savepoint)
   */
  public void rollback(Savepoint savepoint) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#releaseSavepoint(java.sql.Savepoint)
   */
  public void releaseSavepoint(Savepoint savepoint) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#createStatement(int, int, int)
   */
  public Statement createStatement(int resultSetType, int resultSetConcurrency,
      int resultSetHoldability) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#prepareStatement(java.lang.String, int, int, int)
   */
  public PreparedStatement prepareStatement(String sql, int resultSetType,
      int resultSetConcurrency, int resultSetHoldability) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#prepareCall(java.lang.String, int, int, int)
   */
  public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency,
      int resultSetHoldability) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#prepareStatement(java.lang.String, int)
   */
  public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#prepareStatement(java.lang.String, int[])
   */
  public PreparedStatement prepareStatement(String sql, int[] columnIndexes) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.Connection#prepareStatement(java.lang.String,
   *      java.lang.String[])
   */
  public PreparedStatement prepareStatement(String sql, String[] columnNames) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

}
