/*
 * WrapperPreparedStatement.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.jdbc;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.Date;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Vector;

/**
 * @author Radu<br>
 * 
 * Created on 06.03.2004
 */
public class WrapperPreparedStatement implements PreparedStatement {

  private String sqlQuery = null;

  private Statement statement = null;

  private Vector parameters = new Vector(5);

  /**
   * @param statement
   *          The Statement to Wrap
   * @param sqlQuery
   *          the query to complete
   */
  public WrapperPreparedStatement(Statement statement, String sqlQuery) {
    this.statement = statement;
    this.sqlQuery = sqlQuery;
  }

  private void addParameters() {
    StringBuffer buffer = new StringBuffer();
    String[] matched = sqlQuery.split("\\x3F");
    for (int i = 0; i < matched.length - 1; i++) {
      buffer.append(matched[i]);
      buffer.append(parameters.get(i));
    }
    buffer.append(matched[matched.length - 1]);
    try {
      buffer.append(parameters.get(matched.length - 1));
    } catch (ArrayIndexOutOfBoundsException ex) {
    }
    sqlQuery = buffer.toString();
  }

  /**
   * @see java.sql.PreparedStatement#executeQuery()
   */
  public ResultSet executeQuery() throws SQLException {
    addParameters();
    return statement.executeQuery(sqlQuery);
  }

  /**
   * @see java.sql.PreparedStatement#executeUpdate()
   */
  public int executeUpdate() throws SQLException {
    addParameters();
    return statement.executeUpdate(sqlQuery);
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setNull(int, int)
   */
  public void setNull(int parameterIndex, int sqlType) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * @see java.sql.PreparedStatement#setBoolean(int, boolean)
   */
  public void setBoolean(int parameterIndex, boolean x) throws SQLException {
    parameters.add(parameterIndex - 1, "'" + x + "'");
  }

  /**
   * @see java.sql.PreparedStatement#setByte(int, byte)
   */
  public void setByte(int parameterIndex, byte x) throws SQLException {
    parameters.add(parameterIndex - 1, new Byte(x));
  }

  /**
   * @see java.sql.PreparedStatement#setShort(int, short)
   */
  public void setShort(int parameterIndex, short x) throws SQLException {
    parameters.add(parameterIndex - 1, new Short(x));
  }

  /**
   * @see java.sql.PreparedStatement#setInt(int, int)
   */
  public void setInt(int parameterIndex, int x) throws SQLException {
    parameters.add(parameterIndex - 1, new Integer(x));
  }

  /**
   * @see java.sql.PreparedStatement#setLong(int, long)
   */
  public void setLong(int parameterIndex, long x) throws SQLException {
    parameters.add(parameterIndex - 1, new Long(x));
  }

  /**
   * @see java.sql.PreparedStatement#setFloat(int, float)
   */
  public void setFloat(int parameterIndex, float x) throws SQLException {
    parameters.add(parameterIndex - 1, new Float(x));
  }

  /**
   * @see java.sql.PreparedStatement#setDouble(int, double)
   */
  public void setDouble(int parameterIndex, double x) throws SQLException {
    parameters.add(parameterIndex - 1, new Double(x));
  }

  /**
   * @see java.sql.PreparedStatement#setBigDecimal(int, java.math.BigDecimal)
   */
  public void setBigDecimal(int parameterIndex, BigDecimal x) throws SQLException {
    parameters.add(parameterIndex - 1, x);
  }

  /**
   * @see java.sql.PreparedStatement#setString(int, java.lang.String)
   */
  public void setString(int parameterIndex, String x) throws SQLException {
    parameters.add(parameterIndex - 1, "'" + x + "'");
  }

  /**
   * @see java.sql.PreparedStatement#setBytes(int, byte[])
   */
  public void setBytes(int parameterIndex, byte[] x) throws SQLException {
    parameters.add(parameterIndex - 1, x);
  }

  /**
   * @see java.sql.PreparedStatement#setDate(int, java.sql.Date)
   */
  public void setDate(int parameterIndex, Date x) throws SQLException {
    parameters.add(parameterIndex - 1, "'" + x + "'");
  }

  /**
   * @see java.sql.PreparedStatement#setTime(int, java.sql.Time)
   */
  public void setTime(int parameterIndex, Time x) throws SQLException {
    parameters.add(parameterIndex - 1, "'" + x + "'");
  }

  /**
   * @see java.sql.PreparedStatement#setTimestamp(int, java.sql.Timestamp)
   */
  public void setTimestamp(int parameterIndex, Timestamp x) throws SQLException {
    parameters.add(parameterIndex - 1, "'" + x + "'");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setAsciiStream(int, java.io.InputStream,
   *      int)
   */
  public void setAsciiStream(int parameterIndex, InputStream x, int length) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setUnicodeStream(int, java.io.InputStream,
   *      int)
   */
  public void setUnicodeStream(int parameterIndex, InputStream x, int length) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setBinaryStream(int, java.io.InputStream,
   *      int)
   */
  public void setBinaryStream(int parameterIndex, InputStream x, int length) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * @see java.sql.PreparedStatement#clearParameters()
   */
  public void clearParameters() throws SQLException {
    parameters.clear();
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setObject(int, java.lang.Object, int, int)
   */
  public void setObject(int parameterIndex, Object x, int targetSqlType, int scale)
      throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setObject(int, java.lang.Object, int)
   */
  public void setObject(int parameterIndex, Object x, int targetSqlType) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setObject(int, java.lang.Object)
   */
  public void setObject(int parameterIndex, Object x) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * @see java.sql.PreparedStatement#execute()
   */
  public boolean execute() throws SQLException {
    return statement.execute(sqlQuery);
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#addBatch()
   */
  public void addBatch() throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setCharacterStream(int, java.io.Reader,
   *      int)
   */
  public void setCharacterStream(int parameterIndex, Reader reader, int length) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setRef(int, java.sql.Ref)
   */
  public void setRef(int i, Ref x) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setBlob(int, java.sql.Blob)
   */
  public void setBlob(int i, Blob x) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setClob(int, java.sql.Clob)
   */
  public void setClob(int i, Clob x) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setArray(int, java.sql.Array)
   */
  public void setArray(int i, Array x) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * @see java.sql.PreparedStatement#getMetaData()
   */
  public ResultSetMetaData getMetaData() throws SQLException {
    return statement.getResultSet().getMetaData();
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setDate(int, java.sql.Date,
   *      java.util.Calendar)
   */
  public void setDate(int parameterIndex, Date x, Calendar cal) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setTime(int, java.sql.Time,
   *      java.util.Calendar)
   */
  public void setTime(int parameterIndex, Time x, Calendar cal) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setTimestamp(int, java.sql.Timestamp,
   *      java.util.Calendar)
   */
  public void setTimestamp(int parameterIndex, Timestamp x, Calendar cal) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#setNull(int, int, java.lang.String)
   */
  public void setNull(int paramIndex, int sqlType, String typeName) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * @see java.sql.PreparedStatement#setURL(int, java.net.URL)
   */
  public void setURL(int parameterIndex, URL x) throws SQLException {
    parameters.add(parameterIndex, "'" + x + "'");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.PreparedStatement#getParameterMetaData()
   */
  public ParameterMetaData getParameterMetaData() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * @see java.sql.Statement#executeQuery(java.lang.String)
   */
  public ResultSet executeQuery(String sql) throws SQLException {
    return statement.executeQuery(sqlQuery);
  }

  /**
   * @see java.sql.Statement#executeUpdate(java.lang.String)
   */
  public int executeUpdate(String sql) throws SQLException {
    return statement.executeUpdate(sqlQuery);
  }

  /**
   * @see java.sql.Statement#close()
   */
  public void close() throws SQLException {
    statement.close();
  }

  /**
   * @see java.sql.Statement#getMaxFieldSize()
   */
  public int getMaxFieldSize() throws SQLException {
    return statement.getMaxFieldSize();
  }

  /**
   * @see java.sql.Statement#setMaxFieldSize(int)
   */
  public void setMaxFieldSize(int max) throws SQLException {
    statement.setMaxFieldSize(max);
  }

  /**
   * @see java.sql.Statement#getMaxRows()
   */
  public int getMaxRows() throws SQLException {
    return statement.getMaxRows();
  }

  /**
   * @see java.sql.Statement#setMaxRows(int)
   */
  public void setMaxRows(int max) throws SQLException {
    statement.setMaxRows(max);
  }

  /**
   * @see java.sql.Statement#setEscapeProcessing(boolean)
   */
  public void setEscapeProcessing(boolean enable) throws SQLException {
    statement.setEscapeProcessing(enable);
  }

  /**
   * @see java.sql.Statement#getQueryTimeout()
   */
  public int getQueryTimeout() throws SQLException {
    return statement.getQueryTimeout();
  }

  /**
   * @see java.sql.Statement#setQueryTimeout(int)
   */
  public void setQueryTimeout(int seconds) throws SQLException {
    statement.setQueryTimeout(seconds);
  }

  /**
   * @see java.sql.Statement#cancel()
   */
  public void cancel() throws SQLException {
    statement.cancel();
  }

  /**
   * @see java.sql.Statement#getWarnings()
   */
  public SQLWarning getWarnings() throws SQLException {
    return statement.getWarnings();
  }

  /**
   * @see java.sql.Statement#clearWarnings()
   */
  public void clearWarnings() throws SQLException {
    statement.clearWarnings();
  }

  /**
   * @see java.sql.Statement#setCursorName(java.lang.String)
   */
  public void setCursorName(String name) throws SQLException {
    statement.setCursorName(name);
  }

  /**
   * @see java.sql.Statement#execute(java.lang.String)
   */
  public boolean execute(String sql) throws SQLException {
    return statement.execute(sql);
  }

  /**
   * @see java.sql.Statement#getResultSet()
   */
  public ResultSet getResultSet() throws SQLException {
    return statement.getResultSet();
  }

  /**
   * @see java.sql.Statement#getUpdateCount()
   */
  public int getUpdateCount() throws SQLException {
    return statement.getUpdateCount();
  }

  /**
   * @see java.sql.Statement#getMoreResults()
   */
  public boolean getMoreResults() throws SQLException {
    return statement.getMoreResults();
  }

  /**
   * @see java.sql.Statement#setFetchDirection(int)
   */
  public void setFetchDirection(int direction) throws SQLException {
    statement.setFetchDirection(direction);
  }

  /**
   * @see java.sql.Statement#getFetchDirection()
   */
  public int getFetchDirection() throws SQLException {
    return statement.getFetchDirection();
  }

  /**
   * @see java.sql.Statement#setFetchSize(int)
   */
  public void setFetchSize(int rows) throws SQLException {
    statement.setFetchSize(rows);
  }

  /**
   * @see java.sql.Statement#getFetchSize()
   */
  public int getFetchSize() throws SQLException {
    return statement.getFetchSize();
  }

  /**
   * @see java.sql.Statement#getResultSetConcurrency()
   */
  public int getResultSetConcurrency() throws SQLException {
    return statement.getResultSetConcurrency();
  }

  /**
   * @see java.sql.Statement#getResultSetType()
   */
  public int getResultSetType() throws SQLException {
    return statement.getResultSetType();
  }

  /**
   * @see java.sql.Statement#addBatch(java.lang.String)
   */
  public void addBatch(String sql) throws SQLException {
    statement.addBatch(sql);
  }

  /**
   * @see java.sql.Statement#clearBatch()
   */
  public void clearBatch() throws SQLException {
    statement.clearBatch();
  }

  /**
   * @see java.sql.Statement#executeBatch()
   */
  public int[] executeBatch() throws SQLException {
    return statement.executeBatch();
  }

  /**
   * @see java.sql.Statement#getConnection()
   */
  public Connection getConnection() throws SQLException {
    return statement.getConnection();
  }

  /**
   * @see java.sql.Statement#getMoreResults(int)
   */
  public boolean getMoreResults(int current) throws SQLException {
    return statement.getMoreResults(current);
  }

  /**
   * @see java.sql.Statement#getGeneratedKeys()
   */
  public ResultSet getGeneratedKeys() throws SQLException {
    return statement.getGeneratedKeys();
  }

  /**
   * @see java.sql.Statement#executeUpdate(java.lang.String, int)
   */
  public int executeUpdate(String sql, int autoGeneratedKeys) throws SQLException {
    return statement.executeUpdate(sql, autoGeneratedKeys);
  }

  /**
   * @see java.sql.Statement#executeUpdate(java.lang.String, int[])
   */
  public int executeUpdate(String sql, int[] columnIndexes) throws SQLException {
    return statement.executeUpdate(sql, columnIndexes);
  }

  /**
   * @see java.sql.Statement#executeUpdate(java.lang.String,
   *      java.lang.String[])
   */
  public int executeUpdate(String sql, String[] columnNames) throws SQLException {
    return statement.executeUpdate(sql, columnNames);
  }

  /**
   * @see java.sql.Statement#execute(java.lang.String, int)
   */
  public boolean execute(String sql, int autoGeneratedKeys) throws SQLException {
    return statement.execute(sql, autoGeneratedKeys);
  }

  /**
   * @see java.sql.Statement#execute(java.lang.String, int[])
   */
  public boolean execute(String sql, int[] columnIndexes) throws SQLException {
    return statement.execute(sql, columnIndexes);
  }

  /**
   * @see java.sql.Statement#execute(java.lang.String, java.lang.String[])
   */
  public boolean execute(String sql, String[] columnNames) throws SQLException {
    return statement.execute(sql, columnNames);
  }

  /**
   * @see java.sql.Statement#getResultSetHoldability()
   */
  public int getResultSetHoldability() throws SQLException {
    return statement.getResultSetHoldability();
  }

}
