/*
 * WrapperStatement.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.jdbc;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;

import ro.raducora.connection.client.AdapterBase;
import ro.raducora.utils.protocol.ConnectInfo;

/**
 * @author Radu<br>
 * 
 * Created on 06.03.2004
 */
public class WrapperStatement implements Statement {

  private AdapterBase clientAdapter = null;

  private ConnectInfo info = null;

  private ResultSet resultSet = null;

  private int updateCount = 0;

  private boolean isConnected;

  /**
   * Inits the wrapper with the AdapterBase
   * 
   * @param clientAdapter
   *          the Adapter used to connect
   */
  public WrapperStatement(AdapterBase clientAdapter, ConnectInfo info) {
    this.clientAdapter = clientAdapter;
    this.info = info;
    isConnected = true;
  }

  /**
   * @see java.sql.Statement#executeQuery(java.lang.String)
   */
  public ResultSet executeQuery(String sql) throws SQLException {
    Object ret = null;
    if (!isConnected) { throw new SQLException("Cannot reopen connection"); }
    try {
      ret = clientAdapter.queryObject(sql);
    } catch (Exception e) {
      throw new SQLException(e.getMessage());
    }
    if (ret instanceof ResultSet) {
      updateCount = 0;
      resultSet = (ResultSet) ret;
      return resultSet;
    }
    throw new SQLException("Please use 'executeUpdate' for update sql statements.");
  }

  /**
   * @see java.sql.Statement#executeUpdate(java.lang.String)
   */
  public int executeUpdate(String sql) throws SQLException {
    Object ret = null;
    if (!isConnected) { throw new SQLException("Cannot reopen connection"); }
    try {
      ret = clientAdapter.queryObject(sql);
    } catch (Exception e) {
      throw new SQLException(e.getMessage());
    }
    if (ret instanceof Integer) {
      updateCount = ((Integer) ret).intValue();
      resultSet = null;
      return updateCount;
    }
    throw new SQLException("This Shouldn't be reached");
  }

  /**
   * @see java.sql.Statement#close()
   */
  public void close() throws SQLException {
    isConnected = false;
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#getMaxFieldSize()
   */
  public int getMaxFieldSize() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#setMaxFieldSize(int)
   */
  public void setMaxFieldSize(int max) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#getMaxRows()
   */
  public int getMaxRows() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#setMaxRows(int)
   */
  public void setMaxRows(int max) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#setEscapeProcessing(boolean)
   */
  public void setEscapeProcessing(boolean enable) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#getQueryTimeout()
   */
  public int getQueryTimeout() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#setQueryTimeout(int)
   */
  public void setQueryTimeout(int seconds) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#cancel()
   */
  public void cancel() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#getWarnings()
   */
  public SQLWarning getWarnings() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#clearWarnings()
   */
  public void clearWarnings() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#setCursorName(java.lang.String)
   */
  public void setCursorName(String name) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * @see java.sql.Statement#execute(java.lang.String)
   */
  public boolean execute(String sql) throws SQLException {
    Object ret = null;
    if (!isConnected) { throw new SQLException("Cannot reopen connection"); }
    try {
      ret = clientAdapter.queryObject(sql);
    } catch (Exception e) {
      throw new SQLException(e.getMessage());
    }
    if (ret instanceof ResultSet) {
      updateCount = 0;
      resultSet = (ResultSet) ret;
      return true;
    }
    if (ret instanceof Integer) {
      resultSet = null;
      updateCount = ((Integer) ret).intValue();
      return true;
    }
    return false;
  }

  /**
   * @see java.sql.Statement#getResultSet()
   */
  public ResultSet getResultSet() throws SQLException {
    return resultSet;
  }

  /**
   * @see java.sql.Statement#getUpdateCount()
   */
  public int getUpdateCount() throws SQLException {
    return updateCount;
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#getMoreResults()
   */
  public boolean getMoreResults() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#setFetchDirection(int)
   */
  public void setFetchDirection(int direction) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#getFetchDirection()
   */
  public int getFetchDirection() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#setFetchSize(int)
   */
  public void setFetchSize(int rows) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#getFetchSize()
   */
  public int getFetchSize() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#getResultSetConcurrency()
   */
  public int getResultSetConcurrency() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#getResultSetType()
   */
  public int getResultSetType() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#addBatch(java.lang.String)
   */
  public void addBatch(String sql) throws SQLException {
    throw new SQLException("Method Not Supported");

  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#clearBatch()
   */
  public void clearBatch() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#executeBatch()
   */
  public int[] executeBatch() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#getConnection()
   */
  public Connection getConnection() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#getMoreResults(int)
   */
  public boolean getMoreResults(int current) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#getGeneratedKeys()
   */
  public ResultSet getGeneratedKeys() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#executeUpdate(java.lang.String, int)
   */
  public int executeUpdate(String sql, int autoGeneratedKeys) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#executeUpdate(java.lang.String, int[])
   */
  public int executeUpdate(String sql, int[] columnIndexes) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#executeUpdate(java.lang.String,
   *      java.lang.String[])
   */
  public int executeUpdate(String sql, String[] columnNames) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#execute(java.lang.String, int)
   */
  public boolean execute(String sql, int autoGeneratedKeys) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#execute(java.lang.String, int[])
   */
  public boolean execute(String sql, int[] columnIndexes) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#execute(java.lang.String, java.lang.String[])
   */
  public boolean execute(String sql, String[] columnNames) throws SQLException {
    throw new SQLException("Method Not Supported");
  }

  /**
   * Method Not Supported
   * 
   * @see java.sql.Statement#getResultSetHoldability()
   */
  public int getResultSetHoldability() throws SQLException {
    throw new SQLException("Method Not Supported");
  }

}
