/*
 * Driver.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.jdbcr;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.DriverPropertyInfo;
import java.sql.SQLException;
import java.util.NoSuchElementException;
import java.util.Properties;
import java.util.StringTokenizer;

import ro.raducora.connection.client.tcp.ClientTCPDAdapter;
import ro.raducora.utils.encryption.key.RC4EncryptionKey;
import ro.raducora.utils.protocol.ConnectInfo;

/**
 * @author Radu<br>
 * 
 * Created on 07.03.2004
 */
public class Driver implements java.sql.Driver {

  static {
    try {
      DriverManager.registerDriver(new Driver());
    } catch (SQLException E) {
      throw new RuntimeException("Can't register driver!");
    }
  }

  /**
   * This driver connects to the given database and makes replicas of all
   * operations in another database. If the database name is "test", the name
   * of the replica will be "test_repl". The 2 databases must have identical
   * internal structure,username and password access but can have different
   * types.
   * 
   * @throws SQLException
   */
  public Driver() throws SQLException {
  }

  /**
   * @see java.sql.Driver#connect(java.lang.String, java.util.Properties)
   */
  public Connection connect(String url, Properties info) throws SQLException {
    if (!parseUrl(url, info)) { return null; }
    return makeConnection(info);
  }

  /**
   * @see java.sql.Driver#acceptsURL(java.lang.String)
   */
  public boolean acceptsURL(String url) throws SQLException {
    return parseUrl(url, new Properties());
  }

  /**
   * @see java.sql.Driver#getPropertyInfo(java.lang.String,
   *      java.util.Properties)
   */
  public DriverPropertyInfo[] getPropertyInfo(String url, Properties info) throws SQLException {
    DriverPropertyInfo result[] = new DriverPropertyInfo[0];
    return result;
  }

  /**
   * @see java.sql.Driver#getMajorVersion()
   */
  public int getMajorVersion() {
    return 1;
  }

  /**
   * @see java.sql.Driver#getMinorVersion()
   */
  public int getMinorVersion() {
    return 1;
  }

  /**
   * @see java.sql.Driver#jdbcCompliant()
   */
  public boolean jdbcCompliant() {
    return false;
  }

  private boolean parseUrl(String url, Properties result) {
    String tmpUrl = url;
    if (tmpUrl.startsWith("jdbc:jabsr://")) {
      tmpUrl = tmpUrl.substring("jdbc:jabs://".length());
      try {
        StringTokenizer tokenizer = new StringTokenizer(tmpUrl, ":/;", true);
        String host = "localhost";
        String port = "11111";
        String database = "";
        host = tokenizer.nextToken();
        if (tokenizer.hasMoreTokens()) {
          String tmp = tokenizer.nextToken();
          if (tmp.equals(":")) {
            port = tokenizer.nextToken();
            if (tokenizer.hasMoreTokens()) tmp = tokenizer.nextToken();
          }
          if (tokenizer.hasMoreTokens() && tmp.equals("/")) {
            database = tokenizer.nextToken();
            if (tokenizer.hasMoreTokens()) tmp = tokenizer.nextToken();
          }
          for (; tmp.equals(";"); tmp = tokenizer.nextToken()) {
            String extra = tokenizer.nextToken();
            StringTokenizer tok2 = new StringTokenizer(extra, "=", false);
            String key = tok2.nextToken().toUpperCase();
            if (tok2.hasMoreTokens()) result.put(key, tok2.nextToken());
            if (!tokenizer.hasMoreTokens()) break;
          }

        }
        if (!tokenizer.hasMoreTokens()) {
          result.put("HOST", host);
          result.put("PORT", port);
          result.put("DBNAME", database);
        } else {
          return false;
        }
        return true;
      } catch (NoSuchElementException nosuchelementexception) {
        return false;
      }
    } else {
      return false;
    }
  }

  private Connection makeConnection(Properties props) throws SQLException {
    String host = null;
    int port = 0;
    String database = null;
    host = props.getProperty("HOST");
    port = Integer.parseInt(props.getProperty("PORT"));
    database = props.getProperty("DBNAME").trim();
    String user = props.getProperty("USER");
    String password = props.getProperty("PASSWORD");
    if (user == null) {
      user = props.getProperty("USER".toLowerCase());
      if (user == null) throw new SQLException("Need a username.");
      props.put("USER", user);
    }
    if (password == null) {
      password = props.getProperty("PASSWORD".toLowerCase());
      if (password == null) throw new SQLException("Need a password.");
      props.put("PASSWORD", password);
    }
    return new WrapperConnection(new ClientTCPDAdapter(host, port, new RC4EncryptionKey(),
        new RC4EncryptionKey()), new ClientTCPDAdapter(host, port, new RC4EncryptionKey(),
        new RC4EncryptionKey()), new ConnectInfo(database, user, password), new ConnectInfo(
        database + "_repl", user, password));
  }

}
