/*
 * ClientDBProps.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.props;

/**
 * @author Radu Coravu
 *  
 */
public interface ClientDBProps extends Props {

  /**
   * Gets Real DB Name
   * 
   * @param dbname
   *          The key to use in search
   * @return the real db name
   */
  public String getRealDBName(String dbname);

  /**
   * Gets the Driver Class Name
   * 
   * @param dbname
   *          the key to search
   * @return the Driver Class Name of form "org.gjt.Driver"
   */
  public String getDriverClassName(String dbname);

  /**
   * Gets Driver Jar Path
   * 
   * @param dbname
   *          the key to search
   * @return Path to the Jar containing the driver ,for example "e:/lib/" or ""
   *         if jar file not needed or in current directory
   */
  public String getDriverJarPath(String dbname);

  /**
   * Gets Driver Jar Filename
   * 
   * @param dbname
   *          Key to search
   * @return File name of jar containing driver class
   */
  public String getDriverJarFilename(String dbname);

  /**
   * Gets Server Subprotocol for connection to the specific database
   * 
   * @param dbname
   *          Key to search
   * @return The server subprotocol ,for example:mysql,pgsql,odbc,mssql
   */
  public String getServerSubprotocol(String dbname);

  /**
   * Gets Server Hostname
   * 
   * @param dbname
   *          Key to search
   * 
   * @return the <code>server hostname</code> to connect to (of type
   *         ip_addrs,ip_addrs:port,dns_name,localhost)
   */
  public String getServerHostname(String dbname);

  /**
   * Gets the minimum number of opened pool connetions for the specific key
   * 
   * @param dbname
   *          Key to search
   * @return min no of connections in the connection pool for the specific
   *         database key
   */
  public int getMinNoConn(String dbname);

  /**
   * Gets the maximum number of opened pool connetions for the specific key
   * 
   * @param dbname
   *          Key to search
   * @return max no of connections in the connection pool for the specific
   *         database key
   */
  public int getMaxNoConn(String dbname);

}
