/*
 * ClientTCPProperties.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.props.properties;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;

import ro.raducora.utils.props.ClientTCPProps;
import ro.raducora.utils.props.Resolver;

/**
 * @author Radu Coravu
 *  
 */

public class ClientTCPProperties implements ClientTCPProps {

  private String propertiesFilename = "./conf/ClientTCP.properties";

  private static String DEFAULT_SERVER_IP = null;

  private static int DEFAULT_SERVER_PORT = 0;

  private PropertyResourceBundle bundle = null;

  /**
   * Constructor. Default properties filename path
   * "./conf/ClientTCP.properties"
   */
  public ClientTCPProperties() throws IOException {
    reload();
  }

  /**
   * Constructor.
   * 
   * @param filename
   *          properties filename(absolute path)
   */
  public ClientTCPProperties(String filename) throws IOException {
    propertiesFilename = filename;
    reload();
  }

  /**
   * Get Server IP.
   */
  public String getServerIP(String dbname) {
    String ret = null;
    try {
      ret = bundle.getString(dbname + ".SERVER_IP");
    } catch (MissingResourceException ex) {
      return DEFAULT_SERVER_IP;
    }
    return ret;
  }

  /**
   * Get Server PORT.
   */
  public int getServerPORT(String dbname) {
    int ret;
    try {
      ret = Integer.parseInt(bundle.getString(dbname + ".SERVER_PORT"));
    } catch (MissingResourceException ex) {
      return DEFAULT_SERVER_PORT;
    }
    return ret;
  }

  /**
   * Reload from file.
   */
  public void reload() throws IOException {
    bundle = new PropertyResourceBundle(new BufferedInputStream(Resolver
        .solveSource(propertiesFilename)));

    DEFAULT_SERVER_IP = bundle.getString("DEFAULT_SERVER_IP");
    DEFAULT_SERVER_PORT = Integer.parseInt(bundle.getString("DEFAULT_SERVER_PORT"));
  }

  //    public static void main(String[] args) throws InterruptedException {
  //        ClientTCPProperties clientprops = new ClientTCPProperties();
  //        System.out.println(clientprops.getServerIP("shop"));
  //        System.out.println(clientprops.getServerPORT("shop"));
  //        Thread.sleep(20000);
  //        clientprops.reload();
  //        System.out.println(clientprops.getServerIP("shop"));
  //        System.out.println(clientprops.getServerPORT("shop"));
  //        //System.out.println(clientprops.getServerIP("sho"));
  //        //System.out.println(clientprops.getServerIP("sho2"));
  //    }

}
