/*
 * ServerProperties.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.props.properties;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;

import ro.raducora.utils.props.Resolver;
import ro.raducora.utils.props.ServerProps;

/**
 * @author Radu<br>
 * 
 * Created on 09.05.2004
 */
public class ServerProperties implements ServerProps {

  private String propertiesFilename = "./conf/Server.properties";

  private PropertyResourceBundle bundle = null;

  /**
   * Constructor. Default properties filename path
   * "./conf/ClientTCP.properties"
   */
  public ServerProperties() throws IOException {
    reload();
  }

  /**
   * Constructor.
   * 
   * @param filename
   *          properties filename(absolute path)
   */
  public ServerProperties(String filename) throws IOException {
    propertiesFilename = filename;
    reload();
  }

  ///**
  // * Gets Output Compression Flag
  // * 
  // * @return true if output needs to be compressed
  // */
  //public boolean compressOutput() {
  //  String ret = null;
  //  try {
  //    ret = bundle.getString("SERVER_COMPRESS_OUTPUT");
  //  } catch (MissingResourceException ex) {
  //   return false;
  //  }
  //  return ret.equalsIgnoreCase("Yes") ? true : false;
  //}

  /**
   * Allow direct connections from clients that found the server through
   * another server.
   * 
   * @return true if direct connection is allowed
   */
  public boolean allowDirectConnection() {
    String ret = null;
    try {
      ret = bundle.getString("SERVER_ALLOW_DIRECT_CONNECTION");
    } catch (MissingResourceException ex) {
      return false;
    }
    return ret.equalsIgnoreCase("Yes") ? true : false;
  }

  /**
   * Gets the max number of allowed clients
   * 
   * @return the max number of allowed connections
   */
  public int getMaxNumberOfAllowedClients() {
    int ret = 111;
    try {
      ret = Integer.parseInt(bundle.getString("SERVER_MAX_NO_ALLOWED_CLIENTS"));
    } catch (MissingResourceException ex) {
    }
    return ret;
  }

  /**
   * Reloads the Properties from properties Source
   */
  public void reload() throws IOException {
    bundle = new PropertyResourceBundle(new BufferedInputStream(Resolver
        .solveSource(propertiesFilename)));
  }

  /*
   * Test
   */
  /*
   * public static void main(String[] args) throws InterruptedException,
   * IOException { ServerProperties serverprops = new ServerProperties();
   * System.out.println(serverprops.allowDirectConnection());
   * System.out.println(serverprops.compressOutput());
   * System.out.println(serverprops.getMaxNumberOfAllowedClients());
   */

}
