/*
 * ClientDBXMLProperties.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.props.xml;

import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import ro.raducora.utils.props.ClientDBProps;
import ro.raducora.utils.props.Resolver;

/**
 * @author Radu Coravu
 *  
 */

public class ClientDBXMLProperties implements ClientDBProps {

  private String propertiesFilename = "./conf/ClientDB.xml";

  private static String DEFAULT_DB_SERVER_HOSTNAME = null;

  private static String DEFAULT_DB_SERVER_SUBPROTOCOL = null;

  private static String DEFAULT_DRIVER_CLASS_NAME = null;

  private static String DEFAULT_DRIVER_JAR_PATH = null;

  private static String DEFAULT_DRIVER_JAR_NAME = null;

  private static int DEFAULT_MIN_NO_CONN = 0;

  private static int DEFAULT_MAX_NO_CONN = 0;

  Document doc = null;

  /**
   * Default Constructor takes as default properties filename path <code>"./conf/ClientDB.xml"</code>
   * 
   * @throws ParserConfigurationException
   * @throws SAXException
   * @throws IOException
   */
  public ClientDBXMLProperties() throws ParserConfigurationException, SAXException, IOException {
    reload();
  }

  /**
   * @param filename
   *          <code> the properties filename</code>
   * @throws ParserConfigurationException
   * @throws SAXException
   * @throws IOException
   */
  public ClientDBXMLProperties(String filename) throws ParserConfigurationException, SAXException,
      IOException {
    propertiesFilename = filename;
    reload();
  }

  /**
   * @see ro.raducora.utils.props.ClientDBProps#getRealDBName(java.lang.String)
   */
  public String getRealDBName(String dbname) {
    NodeList list = doc.getElementsByTagName("DATABASE");
    for (int i = 0; i < list.getLength(); i++) {
      if (getAttributeValue(list.item(i), "NAME").equals(dbname))
          return getPCDataChild(getChildByName(list.item(i), "REAL_DATABASE_NAME"));
    }
    return dbname;
  }

  /**
   * @see ro.raducora.utils.props.ClientDBProps#getServerHostname(java.lang.String)
   */
  public String getServerHostname(String dbname) {
    NodeList list = doc.getElementsByTagName("DATABASE");
    for (int i = 0; i < list.getLength(); i++) {
      if (getAttributeValue(list.item(i), "NAME").equals(dbname))
          return getPCDataChild(getChildByName(list.item(i), "DB_SERVER_HOSTNAME"));
    }
    return DEFAULT_DB_SERVER_HOSTNAME;
  }

  /**
   * @see ro.raducora.utils.props.ClientDBProps#getDriverClassName(java.lang.String)
   */
  public String getDriverClassName(String dbname) {
    NodeList list = doc.getElementsByTagName("DATABASE");
    for (int i = 0; i < list.getLength(); i++) {
      if (getAttributeValue(list.item(i), "NAME").equals(dbname))
          return getPCDataChild(getChildByName(list.item(i), "DRIVER_CLASS_NAME"));
    }
    return DEFAULT_DRIVER_CLASS_NAME;
  }

  /**
   * @see ro.raducora.utils.props.ClientDBProps#getDriverJarPath(java.lang.String)
   */
  public String getDriverJarPath(String dbname) {
    NodeList list = doc.getElementsByTagName("DATABASE");
    for (int i = 0; i < list.getLength(); i++) {
      if (getAttributeValue(list.item(i), "NAME").equals(dbname))
          return getPCDataChild(getChildByName(list.item(i), "DRIVER_JAR_PATH"));
    }
    return DEFAULT_DRIVER_JAR_PATH;
  }

  /**
   * @see ro.raducora.utils.props.ClientDBProps#getDriverJarFilename(java.lang.String)
   */
  public String getDriverJarFilename(String dbname) {
    NodeList list = doc.getElementsByTagName("DATABASE");
    for (int i = 0; i < list.getLength(); i++) {
      if (getAttributeValue(list.item(i), "NAME").equals(dbname))
          return getPCDataChild(getChildByName(list.item(i), "DRIVER_JAR_NAME"));
    }
    return DEFAULT_DRIVER_JAR_NAME;
  }

  /**
   * @see ro.raducora.utils.props.ClientDBProps#getServerSubprotocol(java.lang.String)
   */
  public String getServerSubprotocol(String dbname) {
    NodeList list = doc.getElementsByTagName("DATABASE");
    for (int i = 0; i < list.getLength(); i++) {
      if (getAttributeValue(list.item(i), "NAME").equals(dbname))
          return getPCDataChild(getChildByName(list.item(i), "DB_SERVER_SUBPROTOCOL"));
    }
    return DEFAULT_DB_SERVER_SUBPROTOCOL;
  }

  /**
   * @see ro.raducora.utils.props.ClientDBProps#getMinNoConn(java.lang.String)
   */
  public int getMinNoConn(String dbname) {
    NodeList list = doc.getElementsByTagName("DATABASE");
    for (int i = 0; i < list.getLength(); i++) {
      if (getAttributeValue(list.item(i), "NAME").equals(dbname))
          return Integer.parseInt(getPCDataChild(getChildByName(list.item(i), "MIN_NO_CONN")));
    }
    return DEFAULT_MIN_NO_CONN;
  }

  /**
   * @see ro.raducora.utils.props.ClientDBProps#getMaxNoConn(java.lang.String)
   */
  public int getMaxNoConn(String dbname) {
    NodeList list = doc.getElementsByTagName("DATABASE");
    for (int i = 0; i < list.getLength(); i++) {
      if (getAttributeValue(list.item(i), "NAME").equals(dbname))
          return Integer.parseInt(getPCDataChild(getChildByName(list.item(i), "MAX_NO_CONN")));
    }
    return DEFAULT_MAX_NO_CONN;
  }

  /**
   * @see ro.raducora.utils.props.ClientDBProps#reload()
   */
  public void reload() throws ParserConfigurationException, SAXException, IOException {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    //factory.setValidating(true);

    DocumentBuilder builder = factory.newDocumentBuilder();

    doc = builder.parse(Resolver.solveSource(propertiesFilename), propertiesFilename);

    DEFAULT_DB_SERVER_HOSTNAME = getPCDataChild(getChildByName(doc.getElementsByTagName("DEFAULTS")
        .item(0), "DEFAULT_DB_SERVER_HOSTNAME"));

    DEFAULT_DB_SERVER_SUBPROTOCOL = getPCDataChild(getChildByName(doc.getElementsByTagName(
        "DEFAULTS").item(0), "DEFAULT_DB_SERVER_SUBPROTOCOL"));

    DEFAULT_DRIVER_CLASS_NAME = getPCDataChild(getChildByName(doc.getElementsByTagName("DEFAULTS")
        .item(0), "DEFAULT_DRIVER_CLASS_NAME"));

    DEFAULT_DRIVER_JAR_PATH = getPCDataChild(getChildByName(doc.getElementsByTagName("DEFAULTS")
        .item(0), "DEFAULT_DRIVER_JAR_PATH"));

    DEFAULT_DRIVER_JAR_NAME = getPCDataChild(getChildByName(doc.getElementsByTagName("DEFAULTS")
        .item(0), "DEFAULT_DRIVER_JAR_NAME"));

    DEFAULT_MIN_NO_CONN = Integer.parseInt(getPCDataChild(getChildByName(doc.getElementsByTagName(
        "DEFAULTS").item(0), "DEFAULT_MIN_NO_CONN")));

    DEFAULT_MAX_NO_CONN = Integer.parseInt(getPCDataChild(getChildByName(doc.getElementsByTagName(
        "DEFAULTS").item(0), "DEFAULT_MAX_NO_CONN")));
  }

  private Node getChildByName(Node parent, String childName) {
    NodeList list = parent.getChildNodes();
    for (int i = 0; i < list.getLength(); i++) {
      if (list.item(i).getNodeName().equalsIgnoreCase(childName)) { return list.item(i); }
    }
    return null;
  }

  private String getPCDataChild(Node parent) {
    NodeList list = parent.getChildNodes();
    for (int i = 0; i < list.getLength(); i++) {
      if (list.item(i).getNodeName().equalsIgnoreCase("#text")
          && list.item(i).getNodeValue().trim().length() > 0) return list.item(i).getNodeValue();
    }
    return "";
  }

  private String getAttributeValue(Node parent, String attrName) {
    NamedNodeMap list = parent.getAttributes();
    if (list.getNamedItem(attrName) != null) { return list.getNamedItem(attrName).getNodeValue(); }
    return "";
  }

//      public static void main(String[] args) throws
//   ParserConfigurationException,
//              SAXException, IOException, InterruptedException {
//          ClientDBXMLProperties props = new ClientDBXMLProperties();
//          System.out.println("Real DB Name for shop:"
//                  + props.getRealDBName("shop"));
//          System.out.println("Hostname for shop:"
//                  + props.getServerHostname("shop"));
//          System.out.println("Server Subprotocol for shop:"
//                  + props.getServerSubprotocol("shop"));
//          System.out.println("Path to Jar for shop:"
//                  + props.getDriverJarPath("shop"));
//          System.out.println("Jar Filename for shop:"
//                  + props.getDriverJarFilename("shop"));
//          System.out.println("Driver class name for shop:"
//                  + props.getDriverClassName("shop"));
//          System.out.println("Min No of Conn for shop:"
//                  + props.getMinNoConn("shop"));
//          System.out.println("Max No of Conn for shop:"
//                  + props.getMaxNoConn("shop"));
//          System.out.println("\n\n");
//
//          System.out.println("Real DB Name for myForum:"
//              + props.getRealDBName("myForum"));
//      System.out.println("Hostname for myForum:"
//              + props.getServerHostname("myForum"));
//      System.out.println("Server Subprotocol for myForum:"
//              + props.getServerSubprotocol("myForum"));
//      System.out.println("Path to Jar for myForum:"
//              + props.getDriverJarPath("myForum"));
//      System.out.println("Jar Filename for myForum:"
//              + props.getDriverJarFilename("myForum"));
//      System.out.println("Driver class name for myForum:"
//              + props.getDriverClassName("myForum"));
//      System.out.println("Min No of Conn for myForum:"
//              + props.getMinNoConn("myForum"));
//      System.out.println("Max No of Conn for myForum:"
//              + props.getMaxNoConn("myForum"));
//      System.out.println("\n\n");
//
//          
//          System.out.println("Real DB Name for mini:"
//                  + props.getRealDBName("mini"));
//          System.out.println("Hostname for mini:"
//                  + props.getServerHostname("mini"));
//          System.out.println("Server Subprotocol for mini:"
//                  + props.getServerSubprotocol("mini"));
//          System.out.println("Path to Jar for mini:"
//                  + props.getDriverJarPath("mini"));
//          System.out.println("Jar Filename for mini:"
//                  + props.getDriverJarFilename("mini"));
//          System.out.println("Driver class name for mini:"
//                  + props.getDriverClassName("mini"));
//          System.out.println("Min No of Conn for mini:"
//                  + props.getMinNoConn("mini"));
//          System.out.println("Max No of Conn for mini:"
//                  + props.getMaxNoConn("mini"));
//          System.out.println("\n\n");
//  
//          System.out.println("Default Real DB Name for min=min:"
//                  + props.getRealDBName("min"));
//          System.out.println("Default Hostname for default:"
//                  + props.getServerHostname("min"));
//          System.out.println("Default Server Subprotocol for default:"
//                  + props.getServerSubprotocol("min"));
//          System.out.println("Default Path to Jar for default:"
//                  + props.getDriverJarPath("min"));
//          System.out.println("Default Jar Filename for default:"
//                  + props.getDriverJarFilename("min"));
//          System.out.println("Default Driver class name for default:"
//                  + props.getDriverClassName("min"));
//          System.out.println("Default Min No of Conn for default:"
//                  + props.getMinNoConn("min"));
//          System.out.println("Default Max No of Conn for default:"
//                  + props.getMaxNoConn("min"));
//  
//          Thread.sleep(20000);
//          props.reload();
//  
//          System.out.println("Real DB Name for shop:"
//                  + props.getRealDBName("shop"));
//          System.out.println("Hostname for shop:"
//                  + props.getServerHostname("shop"));
//          System.out.println("Server Subprotocol for shop:"
//                  + props.getServerSubprotocol("shop"));
//          System.out.println("Path to Jar for shop:"
//                  + props.getDriverJarPath("shop"));
//          System.out.println("Jar Filename for shop:"
//                  + props.getDriverJarFilename("shop"));
//          System.out.println("Driver class name for shop:"
//                  + props.getDriverClassName("shop"));
//          System.out.println("Min No of Conn for shop:"
//                  + props.getMinNoConn("shop"));
//          System.out.println("Max No of Conn for shop:"
//                  + props.getMaxNoConn("shop"));
//          System.out.println("\n\n");
//  
//          System.out.println("Real DB Name for mini:"
//                  + props.getRealDBName("mini"));
//          System.out.println("Hostname for mini:"
//                  + props.getServerHostname("mini"));
//          System.out.println("Server Subprotocol for mini:"
//                  + props.getServerSubprotocol("mini"));
//          System.out.println("Path to Jar for mini:"
//                  + props.getDriverJarPath("mini"));
//          System.out.println("Jar Filename for mini:"
//                  + props.getDriverJarFilename("mini"));
//          System.out.println("Driver class name for mini:"
//                  + props.getDriverClassName("mini"));
//          System.out.println("Min No of Conn for mini:"
//                  + props.getMinNoConn("mini"));
//          System.out.println("Max No of Conn for mini:"
//                  + props.getMaxNoConn("mini"));
//          System.out.println("\n\n");
//  
//          System.out.println("Default Real DB Name for min=min:"
//                  + props.getRealDBName("min"));
//          System.out.println("Default Hostname for default:"
//                  + props.getServerHostname("min"));
//          System.out.println("Default Server Subprotocol for default:"
//                  + props.getServerSubprotocol("min"));
//          System.out.println("Default Path to Jar for default:"
//                  + props.getDriverJarPath("min"));
//          System.out.println("Default Jar Filename for default:"
//                  + props.getDriverJarFilename("min"));
//          System.out.println("Default Driver class name for default:"
//                  + props.getDriverClassName("min"));
//          System.out.println("Default Min No of Conn for default:"
//                  + props.getMinNoConn("min"));
//          System.out.println("Default Max No of Conn for default:"
//                  + props.getMaxNoConn("min"));
//      }
}
