/*
 * ClientTCPXMLProperties.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.props.xml;

import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import ro.raducora.utils.props.ClientTCPProps;
import ro.raducora.utils.props.Resolver;

/**
 * @author Radu Coravu
 *  
 */

public class ClientTCPXMLProperties implements ClientTCPProps {

  private String propertiesFilename = "./conf/ClientTCP.xml";

  private String DEFAULT_SERVER_IP = null;

  private int DEFAULT_SERVER_PORT = 0;

  private Document doc = null;

  /**
   * Default Constructor takes as default properties filename path <code>"./conf/ClientTCP.xml"</code>
   * 
   * @throws ParserConfigurationException
   * @throws SAXException
   * @throws IOException
   */
  public ClientTCPXMLProperties() throws ParserConfigurationException, SAXException, IOException {
    reload();
  }

  /**
   * @param filename
   *          <code>the properties filename</code>
   * @throws ParserConfigurationException
   * @throws SAXException
   * @throws IOException
   */
  public ClientTCPXMLProperties(String filename) throws ParserConfigurationException, SAXException,
      IOException {
    propertiesFilename = filename;
    reload();
  }

  /**
   * @see ro.raducora.utils.props.ClientTCPProps#getServerIP(java.lang.String)
   */
  public String getServerIP(String dbname) {
    String ret = null;

    NodeList list = doc.getElementsByTagName("DATABASE");
    for (int i = 0; i < list.getLength(); i++) {
      if (getAttributeValue(list.item(i), "NAME").equals(dbname))
          ret = getPCDataChild(getChildByName(list.item(i), "SERVER_IP"));
    }
    if (ret == null) return DEFAULT_SERVER_IP;
    return ret;
  }

  /**
   * @see ro.raducora.utils.props.ClientTCPProps#getServerPORT(java.lang.String)
   */
  public int getServerPORT(String dbname) {
    int ret = 0;
    NodeList list = doc.getElementsByTagName("DATABASE");
    for (int i = 0; i < list.getLength(); i++) {
      if (getAttributeValue(list.item(i), "NAME").equals(dbname))
          ret = Integer.parseInt(getPCDataChild(getChildByName(list.item(i), "SERVER_PORT")));
    }

    if (ret == 0) return DEFAULT_SERVER_PORT;
    return ret;
  }

  /**
   * @see ro.raducora.utils.props.ClientTCPProps#reload()
   */
  public void reload() throws ParserConfigurationException, SAXException, IOException {

    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    //factory.setValidating(true);

    DocumentBuilder builder = factory.newDocumentBuilder();

    doc = builder.parse(Resolver.solveSource(propertiesFilename), propertiesFilename);

    DEFAULT_SERVER_IP = getPCDataChild(getChildByName(doc.getElementsByTagName("DEFAULTS").item(0),
        "DEFAULT_SERVER_IP"));

    DEFAULT_SERVER_PORT = Integer
        .parseInt(getPCDataChild(
            getChildByName(doc.getElementsByTagName("DEFAULTS").item(0), "DEFAULT_SERVER_PORT"))
            .trim());
  }

  private Node getChildByName(Node parent, String childName) {
    NodeList list = parent.getChildNodes();
    for (int i = 0; i < list.getLength(); i++) {
      if (list.item(i).getNodeName().equalsIgnoreCase(childName)) { return list.item(i); }
    }
    return null;
  }

  private String getPCDataChild(Node parent) {
    NodeList list = parent.getChildNodes();
    for (int i = 0; i < list.getLength(); i++) {
      if (list.item(i).getNodeName().equalsIgnoreCase("#text")
          && list.item(i).getNodeValue().trim().length() > 0) return list.item(i).getNodeValue();
    }
    return "";
  }

  private String getAttributeValue(Node parent, String attrName) {
    NamedNodeMap list = parent.getAttributes();
    if (list.getNamedItem(attrName) != null) { return list.getNamedItem(attrName).getNodeValue(); }
    return "";
  }

  //    public static void main(String[] args)
  //        throws ParserConfigurationException, SAXException, IOException,
  // InterruptedException {
  //        ClientTCPXMLProperties clientprops = new ClientTCPXMLProperties();
  //        System.out.println("Shop Server IP:" + clientprops.getServerIP("shop"));
  //        System.out.println("Shop Server PORT:" +
  // clientprops.getServerPORT("shop"));
  //        System.out.println("Mini Server IP:" + clientprops.getServerIP("mini"));
  //        System.out.println("Mini Server PORT:" +
  // clientprops.getServerPORT("mini"));
  //        System.out.println("Default Server IP:" +
  // clientprops.getServerIP("sho"));
  //        System.out.println("Default Server PORT:" +
  // clientprops.getServerPORT("sho2"));
  //        Thread.sleep(20000);
  //        clientprops.reload();
  //        System.out.println("Shop Server IP:" + clientprops.getServerIP("shop"));
  //        System.out.println("Shop Server PORT:" +
  // clientprops.getServerPORT("shop"));
  //        System.out.println("Mini Server IP:" + clientprops.getServerIP("mini"));
  //        System.out.println("Mini Server PORT:" +
  // clientprops.getServerPORT("mini"));
  //        System.out.println("Default Server IP:" +
  // clientprops.getServerIP("sho"));
  //        System.out.println("Default Server PORT:" +
  // clientprops.getServerPORT("sho2"));
  //    }
}
