/*
 * ServerXMLProperties.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.props.xml;

import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import ro.raducora.utils.props.Resolver;
import ro.raducora.utils.props.ServerProps;

/**
 * @author Radu Coravu<br>
 * 
 * Created on May 8, 2004 <br>
 * Server XML Properties Reader
 */
public class ServerXMLProperties implements ServerProps {

  private String propertiesFilename = "./conf/Server.xml";

  private Document doc = null;

  /**
   * Default Constructor takes as default properties filename path <code>"./conf/Server.xml"</code>
   * 
   * @throws ParserConfigurationException
   * @throws SAXException
   * @throws IOException
   */
  public ServerXMLProperties() throws ParserConfigurationException, SAXException, IOException {
    reload();
  }

  /**
   * @param filename
   *          <code> the properties filename</code>
   * @throws ParserConfigurationException
   * @throws SAXException
   * @throws IOException
   */
  public ServerXMLProperties(String filename) throws ParserConfigurationException, SAXException,
      IOException {
    propertiesFilename = filename;
    reload();
  }

//  /**
//   * Return true if output needs to be compressed
//   */
//  public boolean compressOutput() {
//    return doc.getElementsByTagName("OUTPUT").item(0).getAttributes().getNamedItem("COMPRESSION")
//        .getNodeValue().equalsIgnoreCase("Yes");
//  }

  /**
   * Returns true if direct connection to client through redirection is
   * possible.
   */
  public boolean allowDirectConnection() {
    return doc.getElementsByTagName("CONNECTION_ROUTING").item(0).getAttributes().getNamedItem(
        "DIRECT").getNodeValue().equalsIgnoreCase("Yes");
  }

  /**
   * Reloads the properties
   */
  public void reload() throws ParserConfigurationException, SAXException, IOException {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    //factory.setValidating(true);
    DocumentBuilder builder = factory.newDocumentBuilder();
    doc = builder.parse(Resolver.solveSource(propertiesFilename), propertiesFilename);
  }

  /**
   * Gets the max number of allowed clients
   * 
   * @return the max number of allowed connections
   */
  public int getMaxNumberOfAllowedClients() {
    return Integer.parseInt(doc.getElementsByTagName("MAX_ALLOWED_CONNECTIONS").item(0)
        .getAttributes().getNamedItem("NUMBER").getNodeValue().trim());
  }

  /**
   * Test the class
   *//*
  public static void main(String[] args) throws ParserConfigurationException, SAXException,
      IOException {
    ServerProps serverProps = new ServerXMLProperties("./conf/Server.xml");
    System.out.println("Allow direct connection:" + serverProps.allowDirectConnection());
    System.out.println("Allow output compression:" + serverProps.compressOutput());
    System.out.println("Max Number of clients:" + serverProps.getMaxNumberOfAllowedClients());
  }*/
}
