/*
 * ConnectInfo.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.protocol;

import java.io.Serializable;
import java.util.Vector;

/**
 * @author Radu Coravu <br>
 *         Holds the connection related info
 */
public class ConnectInfo implements Serializable {

  private String dbname;

  private String username;

  private String password;

  private Vector path = new Vector(4);

  /**
   * Constructor.
   * 
   * @param dbname
   *          database name
   * @param username
   *          username required for db
   * @param password
   *          password required for db
   */
  public ConnectInfo(String dbname, String username, String password) {
    this.dbname = dbname;
    this.username = username;
    this.password = password;
  }

  /**
   * Returns the database name
   * 
   * @return db name
   */
  public String getDBName() {
    return dbname;
  }

  /**
   * Returns the username
   * 
   * @return username
   */
  public String getUsername() {
    return username;
  }

  /**
   * Returns the password
   * 
   * @return password
   */
  public String getPassword() {
    return password;
  }

  /**
   * Checks if this serverID is part of a loop that is if it has already been
   * encountered
   * 
   * @param serverID
   *          the unique id of encountered server usually IP+":"+PORT ,checks
   *          if already encountered
   * @return true if it is a loop
   */
  public boolean checkLoop(String serverID) {
    return path.contains(serverID);
  }

  /**
   * Adds a serverID to the path the ConnectInfo object is taking
   * 
   * @param serverID
   *          unique serverID to add to path
   */
  public void addToPath(String serverID) {
    path.add(serverID);
  }

  /**
   * Gets the number of servers in path
   * 
   * @return the number of servers in path
   */
  public int getNoServersInPath() {
    return path.size();
  }

  /**
   * To String
   */
  public String toString() {
    return "DBName:" + dbname + " User:" + username + " Pass:" + password;
  }
}
