/*
 * SerializedResultSetMetaData.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.protocol;

import java.io.Serializable;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

/**
 * @author Radu Coravu
 *  
 */
public class SerializedResultSetMetaData implements ResultSetMetaData, Serializable {

  private int columnCount;

  private String columnNames[];

  private String columnClassNames[];

  private String catalogNames[];

  private int columnTypes[];

  /**
   * A serializable copy of an ResultSetMetaData variable
   * 
   * @param rsMetaData
   *          the ResultSetMetaData to wrap
   * @throws SQLException
   */
  public SerializedResultSetMetaData(ResultSetMetaData rsMetaData) throws SQLException {
    columnCount = rsMetaData.getColumnCount();
    columnNames = new String[columnCount];
    columnClassNames = new String[columnCount];
    catalogNames = new String[columnCount];
    columnTypes = new int[columnCount];
    for (int i = 0; i < columnCount; i++) {
      columnNames[i] = rsMetaData.getColumnName(i + 1);
      columnClassNames[i] = rsMetaData.getColumnClassName(i + 1);
      catalogNames[i] = rsMetaData.getCatalogName(i + 1);
      columnTypes[i] = rsMetaData.getColumnType(i + 1);
    }
  }

  /**
   * @see java.sql.ResultSetMetaData#getColumnCount()
   */
  public int getColumnCount() throws SQLException {
    return columnCount;
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#isAutoIncrement(int)
   */
  public boolean isAutoIncrement(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#isCaseSensitive(int)
   */
  public boolean isCaseSensitive(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#isSearchable(int)
   */
  public boolean isSearchable(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#isCurrency(int)
   */
  public boolean isCurrency(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#isNullable(int)
   */
  public int isNullable(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#isSigned(int)
   */
  public boolean isSigned(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#getColumnDisplaySize(int)
   */
  public int getColumnDisplaySize(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#getColumnLabel(int)
   */
  public String getColumnLabel(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * @see java.sql.ResultSetMetaData#getColumnName(int)
   */
  public String getColumnName(int column) throws SQLException {
    return columnNames[column - 1];
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#getSchemaName(int)
   */
  public String getSchemaName(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#getPrecision(int)
   */
  public int getPrecision(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#getScale(int)
   */
  public int getScale(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#getTableName(int)
   */
  public String getTableName(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * @see java.sql.ResultSetMetaData#getCatalogName(int)
   */
  public String getCatalogName(int column) throws SQLException {
    return catalogNames[column - 1];
  }

  /**
   * @see java.sql.ResultSetMetaData#getColumnType(int)
   */
  public int getColumnType(int column) throws SQLException {
    return columnTypes[column - 1];
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#getColumnTypeName(int)
   */
  public String getColumnTypeName(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#isReadOnly(int)
   */
  public boolean isReadOnly(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#isWritable(int)
   */
  public boolean isWritable(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not implemented
   * 
   * @see java.sql.ResultSetMetaData#isDefinitelyWritable(int)
   */
  public boolean isDefinitelyWritable(int column) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * @see java.sql.ResultSetMetaData#getColumnClassName(int)
   */
  public String getColumnClassName(int column) throws SQLException {
    return columnClassNames[column - 1];
  }
}
